#ifndef _PARALLELS_CPP_
#define _PARALLELS_CPP_

#include <iostream.h>

#include <reference.h>
#include <table.h>
#include <event.h>
#include <part.h>
#include <track.h>
#include <scoreTrack.h>
#include <position.h>
#include <song.h>
#include <note.h>
#include <midiEvent.h>

#include "parallels.h"


#define PARALLELS_CONTEXT Addon::TREAT_SONG

extern "C" {
  Addon * parallels(Element * target) { return new Parallels(target); }
  char  * parallels_name() { return "parallels"; }
  char  * parallels_category() { return "harmony"; }
  int     parallels_context() { return PARALLELS_CONTEXT; }
}


Parallels::Parallels(Element * target) : Addon("find parallels", target, TO_BE_UNDONE, PARALLELS_CONTEXT) { run(); }

Parallels::~Parallels() {

}


int sgn(int i) {
  if (i==0) return 0;
  else if (i>0) return 1;
  else return -1;
}

void Parallels::song(Song * _song) {

  int ntracks = 0;
  ntracks = _song->size();

  _track = new ScoreTrack();
  _track->setName("* parallels...");
  _part = new Part(_track);
  _track->add(_part);
  _song->add(_track);

  Track * tr[ntracks];
  Part * pt[ntracks];
  int stp[ntracks];
  int maxvoices = ntracks*10;
  int pitches[maxvoices]; // in here we put the pitches to compare!
  int tendency[maxvoices]; // in here we put the tangent slope prev -> note
  int lengths[maxvoices];
  int count = 0;

  for (int t=0; t<ntracks; t++) {
    tr[t] = (Track*) _song->get(t);
    pt[t] = (Part*) tr[t]->first();
    if (pt[t]) {
      pt[t]->wind(0);
      stp[t] = 0;
    }
  }

  int stop = 0;
  long pp = 0;
  long oldTimePos = 0;
  Note * note;
  MidiEvent * mev;
  Element * el = 0;
  Reference * ref = 0;
  Note * prevnote = 0;
  int pitchDiff = 0;
  int parallel = 0;

  while (stop<2) {
    for (int t=0; t<ntracks; t++) {
      if (pt[t]) {
	for (pt[t]->startMem();pt[t]->moreMem();) {
	  if (pt[t]->memEndsAt(pp)) {
	    note = (Note*) pt[t]->mem();
	    pt[t]->cutMem();
	  } else {
	    pt[t]->incMem();
	  }
	}
      }
    }
    
    count = 0;

    for (int t=0; t<ntracks; t++) {
      if (pt[t]!=0) {
	while (pt[t]->currentStartsAt(pp)) {
	  if (pt[t]->current(NOTE)) {
	    note = (Note*) pt[t]->current();
	    if (count<maxvoices) {
	      prevnote = (Note*) pt[t]->prev(note);
	      if (prevnote!=0) tendency[count] = note->pitch() - prevnote->pitch(); else tendency[count] = 0;
	      lengths[count] = note->length();
	      pitches[count++] = note->pitch();
	    } else {
	      cout << "ERROR: too many voices!" << endl;
	    }
	    pt[t]->rememberCurrent();
	  } else if (pt[t]->current(MIDIEVENT)) {
	    mev = (MidiEvent*) pt[t]->current();
	  } else {
	    cout << "track " << t << *pt[t]->current() << endl;
	  }
	  if (!pt[t]->currentInc()) stp[t]++;
	}
      }
    }

    for (int i=0;i<count;i++) { // check for parallels:
      for (int k=0;k<i;k++) {
	pitchDiff = abs(pitches[i] - pitches[k]);
	parallel = (tendency[i]==tendency[k]) ? 1 : ( sgn(tendency[i])==sgn(tendency[k]) ? 2 : 0 ); // 0: nothing, 1: parallel, 2: verdeckt parallel
	// if (pitches[i] == pitches[k]) cout << "primes at " << pp << endl;
	if (pitchDiff==7) {
	  if (parallel==1) add("qp",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // quint parallels
	  else if (parallel==2) add("vqp",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // verdeckte quint parallels
	}
	if (pitchDiff==12) {
	  if (parallel==1) add("op",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // octave parallels
	  else if (parallel==2) add("vop",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // verdeckte octave parallels
	  }
      }
    }

    stop = 1;
    for (int t=0; t<ntracks; t++) { if (stp[t]==0) stop = 0; }
      
    if (stop==1) {
      stop = 2;
      for (int t=0; t<ntracks; t++) {
	pt[t]->startMem();
	if (pt[t]->moreMem()) stop = 1;
      }
    }
    pp++;
  }
  
  pp += 384;
  // wait(pp*tempo,left*tempo);
  // devman->sync();
  cout << "stop" << endl;
  
  for (int t=0; t<ntracks; t++) {
    pt[t]->startMem();
    if (pt[t]->moreMem()) {
      pt[t]->clearMem();
    }
  }

}

void Parallels::add(const char * text, long pos, int p1, int p2, int len1, int len2, int parallel) {
  _part->add(new Note(p1,87,len1,Position(pos),0,parallel-2));
  _part->add(new Note(p2,87,len2,Position(pos),0,parallel-2));
}


void Parallels::undo() {
}

void Parallels::redo() {
}






#endif
