#ifndef _EVENT_H_
#define _EVENT_H_

#include "atom.h"
#include "position.h"

/** Events are atoms with a well defined position and length
 */
class Event : public Atoma
{
 private:
  Position _pos;
  long _len;

 public:
  /** Default constructor */
  Event();

  /** Constructor used by copy() */
  Event(Event*);

  /** Constructor at a position */
  Event(Position);

  /** Constructor with position and length */
  Event(Position,long);

  /** returns the start position (offset) of an event */
  Position & start();

  /** returns the end position of an event */
  long end();

  /** returns the length of an event */
  long length();

  /** sets the start position */
  void setStart(Position);
  void snapStart(int);

  /** sets the length of an event */
  void setLength(long);

  /** print has to be implemented */
  virtual ostream & print(int,ostream&)=0;

  /** flush has to be implemented */
  virtual void flush(char*)=0;

  /** copy has to be implemented */
  virtual Element * copy()=0;

  bool startsBefore(long);

  bool startsBehind(long);

  bool startsAt(long);
  
  bool endsBefore(long);

  bool endsBehind(long);

  bool endsAt(long);

  virtual bool isEvent() { return true; }

  // static Event * load(char*,ifstream*&);
};

#endif
