#ifndef _NOTE_CPP_
#define _NOTE_CPP_

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <fstream.h>
#include <stdlib.h>
#include "note.h"
#include "song.h"
#include "vector.h"
#include "str.h"
#include "loader.h"
#include "ornament.h"
#include "addOrnament.h"
#include "lyrics.h"
#include "expression.h"
#include "reference.h"
#include "event.h"

// #include "kbNoteExt.h"
// #include "kbLyrics.h"
// #include "kbStem.h"
// #include "kbExp.h"
// #include "kbBow.h"

extern Song * sonG;
extern char * getcmd(char*);

Note::Note( )
  : Event(0,0), _pitch(0),_vel(0), _enh(0), _chan(-1), _ornament(new Vector()), _cPitch(new char[4]) { _type = NOTE; }

Note::Note(Note * n) : Event(n) {
  _type = NOTE;
  if (n) {
    _pitch = n->pitch();
    _vel = n->vel();
    _enh = n->enh();
    _chan = n->chan();
    _ornament = (Vector*) n->_ornament->copy();
    _cPitch = strdup(n->_cPitch);
  }
}

Note::Note(int pitch, int vel, long len, Position pos, int enh, int chan)
  : Event(pos,len), _pitch(pitch),_vel(vel), _enh(enh), _chan(chan), _cPitch(new char[4]) {
  _ornament = new Vector();
  _type = NOTE;
}

Note::Note(char * pitch, int vel, long len, Position pos, int enh, int chan)
  : Event(pos,len), _vel(vel), _enh(enh), _chan(chan), _cPitch(new char[4]) {
  _ornament = new Vector();
  _type = NOTE;
  int o = atoi(pitch+1);
  if (o==0) o = atoi(pitch+2);
  int f = 24;
  switch (pitch[0]) {
  case 'c': f = 24; break;
  case 'd': f = 26; break;
  case 'e': f = 28; break;
  case 'f': f = 29; break;
  case 'g': f = 31; break;
  case 'a': f = 33; break;
  case 'h': f = 35; break;
  case 'b': f = 35; break;
  case 'C': f = 24; break;
  case 'D': f = 26; break;
  case 'E': f = 28; break;
  case 'F': f = 29; break;
  case 'G': f = 31; break;
  case 'A': f = 33; break;
  case 'H': f = 35; break;
  case 'B': f = 35; break;
  }
  switch (pitch[1]) {
  case '#': f+=1; break;
  case 'b': f-=1; break;
  }
  f+=12*o;
  _pitch = f;
}


Note::~Note() {
  delete _ornament;
  delete _cPitch;
}

int Note::pitch() { return _pitch; }

int Note::vel() { return _vel; }

int Note::enh() { return _enh; }

int Note::chan() { return _chan; }

Vector * Note::ornament() { return _ornament; }


void Note::setPitch(int p) { _pitch = p; }

void Note::setVel(int v) { _vel=v; if (_vel>127) _vel = 127; if (_vel<1) _vel = 1; }

void Note::setEnh(int e) { _enh=e; }

void Note::setChan(int c) { _chan=c; }

void Note::add(Ornament * o) {
  _ornament->add(o);
}

void Note::remove(Ornament * o) {
  // no delete ornament here, this is done in AddOrnament's destructor
  _ornament->remove(o);
}

Expression * Note::setExpression(int exp) {
  Expression * ne = 0;
  for (Element * el = _ornament->content(); el != 0; el = Element::next(el)) {
    if (el->isA() == EXPRESSION) if (((Expression*)el)->expression()==exp) ne = (Expression*) el;
  }
  if (ne==0) {
    ne = new Expression(exp);
    sonG->doo(new AddOrnament(ne, this));
  }
  return ne;
}

Bow * Note::setBow(int len, int dir, int delta) {
  Bow * nb = 0;
  for (Element * el = _ornament->content(); el != 0; el = Element::next(el)) {
    if (el->isA() == BOW) nb = (Bow*) el;
  }
  if (nb==0) {
    nb = new Bow(len,dir,delta);
    sonG->doo(new AddOrnament(nb, this));
  }
  return nb;
}

void Note::removeExp() {
  for (int i=0; i<_ornament->size(); ) {
    Element * el = _ornament->get(i);
    if (el->isA() == EXPRESSION) _ornament->remove(el);
    else i++;
  }
}

void Note::removeBow() {
  for (int i=0; i<_ornament->size(); ) {
    Element * el = _ornament->get(i);
    if (el->isA() == BOW) _ornament->remove(el);
    else i++;
  }
}

Lyrics * Note::lyrics() {
  Lyrics * rv = 0;
  for (Element * el = _ornament->content(); el != 0; el = Element::next(el)) {
    if (el->isA() == LYRICS) rv = (Lyrics*) el;
  }
  return rv;
}

char * Note::cPitch() {
  int n = _pitch%12;
  char ff;
  char sg;
  switch (n) {
  case 0: ff = 'c'; sg =' '; break;
  case 1: ff = 'c'; sg ='#'; break;
  case 2: ff = 'd'; sg =' '; break;
  case 3: ff = 'd'; sg ='#'; break;
  case 4: ff = 'e'; sg =' '; break;
  case 5: ff = 'f'; sg =' '; break;
  case 6: ff = 'f'; sg ='#'; break;
  case 7: ff = 'g'; sg =' '; break;
  case 8: ff = 'g'; sg ='#'; break;
  case 9: ff = 'a'; sg =' '; break;
  case 10: ff = 'a'; sg ='#'; break;
  case 11: ff = 'h'; sg =' '; break;
  }
  if (sg==' ')
    sprintf(_cPitch,"%c%d ",ff,(int)_pitch/12-2);
  else
    sprintf(_cPitch,"%c%c%d",ff,sg,(int)_pitch/12-2);
  return _cPitch;
}


/* void Note::sExt(NoteExt * e) { ext = e; }

void Note::appendExt(NoteExt * e) {
  NoteExt * last = 0;
  for (NoteExt * c = ext; c!=0; c=c->next()) last = c;
  if (last==0) ext = e;
  else last->sNext(e);
}

KbExp * Note::giveExp(int i) {
  KbExp * ne = 0;
  NoteExt * last = 0;
  for (NoteExt * c = ext; c!=0; c=c->gNext()) {
    if (c->isExp()) if  (((KbExp*)c)->gExpression()==i) ne = (KbExp*)c;
    last = c;
  }
  if (ne==0) {
    ne = new KbExp(this,i);
    if (last==0) ext = ne;
    else last->sNext(ne);
  }
  return ne;
}

KbBow * Note::giveBow(int len, int dir, int delta) {
  KbBow * nb = 0;
  NoteExt * last = 0;
  for (NoteExt * c = ext; c!=0; c=c->gNext()) {
    if (c->isBow()) nb = (KbBow*)c;
    last = c;
  }
  if (nb==0) {
    nb = new KbBow(this,len,dir,delta);
    if (last==0) ext = nb;
    else last->sNext(nb);
  }
  return nb;
}

KbLyrics * Note::lyrics() {
  KbLyrics * rv = 0;
  for (NoteExt * e = ext; e!=0; e = e->gNext())
    if (e->isLyrics()) rv =(KbLyrics*)e;
  return rv;
}

int Note::stemDir() {
  int rv = -1;
  for (NoteExt * e = ext; e!=0; e = e->gNext())
    if (e->isStem()) rv =((KbStem*)e)->gDir();
  return rv;
}

int Note::expression() {
  int rv = -1;
  for (NoteExt * e = ext; e!=0; e = e->gNext())
    if (e->isExp()) rv =((KbExp*)e)->gExpression();
  return rv;
}

int Note::bowLength() {
  int rv = -1;
  for (NoteExt * e = ext; e!=0; e = e->gNext())
    if (e->isBow()) rv =((KbBow*)e)->gLength();
  return rv;
}

KbExp * Note::gExp(NoteExt * start) {
  KbExp * rv = 0;
  if (start==0) start = ext;
  else start = start->gNext();
  for (NoteExt * e = start; (e!=0)&&(rv==0); e = e->gNext())
    if (e->isExp()) rv =(KbExp*)e;
  return rv;
}

KbBow * Note::gBow() {
  KbBow * rv = 0;
  for (NoteExt * e = ext; e!=0; e = e->gNext())
    if (e->isBow()) rv =(KbBow*)e;
  return rv;
}


void Note::flipStem() {
  KbStem * st = 0;
  for (NoteExt * e = ext; e!=0; e = e->gNext())
    if (e->isStem()) st =(KbStem*)e;
  if (st==0) {
    st = new KbStem(this,StemUp);
    appendExt(st);
  } else st->flip();
}

void Note::flipBow() {
  KbBow * bw = 0;
  for (NoteExt * e = ext; e!=0; e = e->gNext())
    if (e->isBow()) bw =(KbBow*)e;
  if (bw!=0) bw->flip();
}

void Note::removeExp() {
  NoteExt * old = 0;
  NoteExt * rm = 0;
  for (NoteExt * e = ext; e!=0; ) {
    if (e->isExp()) {
      if (old==0) ext = e->gNext();
      else old->sNext(e->gNext());
      rm = e;
      e = e->gNext();
      delete rm;
    } else e = e->gNext();
  }
}

void Note::removeBow() {
  NoteExt * old = 0;
  NoteExt * rm = 0;
  for (NoteExt * e = ext; e!=0; ) {
    if (e->isBow()) {
      if (old==0) ext = e->gNext();
      else old->sNext(e->gNext());
      rm = e;
      e = e->gNext();
      delete rm;
    } else e = e->gNext();
  }
  }*/


ostream & Note::print(int dep, ostream & s) {
  s << spc(dep) << "<NOTE pos=\"" << start().ticks() << "\" len=\"" << length() << "\" pitch=\"" << _pitch << "\" vel=\"" << _vel << "\" enh=\"" << _enh << "\" chan=\"" << _chan << "\" >" << endl;
  /*
  for (NoteExt * e = gExt(); e != 0; e = e->gNext())
    s << *e;
  */
  // s << spc(dep) << *_ornament << endl;
  // _ornament->printContent(dep+2,s);
  for (Element * el = _ornament->content(); el != 0; el = Element::next(el)) {
    el->print(dep+1,s);
  }
  s << spc(dep) << "</NOTE>" << endl;
  return s;
}

void Note::flush(char * c) {
  cout << c << "NOTE at " << start().gBar(0,4,4) << "." << start().gBeat(0,4,4) << "." << start().gTicks(0,4,4) << endl;
}

Element * Note::copy() {
  return new Note(this);
}



Element * Note::load(char * aline, ifstream *& inPtr, Element * parent) {
  Note * note = new Note();
  Table * attr = Loader::getAttributes("NOTE",aline);
  if (attr!=0) {

    int pitch = 0;
    int vel = 0;
    long llen = 0;
    long pos = 0;
    int enh = 0;
    int chan = -1;

    if (attr->getEntry("pitch")) pitch = atoi(((String*) attr->getEntry("pitch"))->getValue());
    if (attr->getEntry("vel")) vel = atoi(((String*) attr->getEntry("vel"))->getValue());
    if (attr->getEntry("len")) llen = atol(((String*) attr->getEntry("len"))->getValue());
    if (attr->getEntry("pos")) pos = atol(((String*) attr->getEntry("pos"))->getValue());
    if (attr->getEntry("enh")) enh = atoi(((String*) attr->getEntry("enh"))->getValue());
    if (attr->getEntry("chan")) chan = atoi(((String*) attr->getEntry("chan"))->getValue());
        
    note->setPitch(pitch);
    note->setVel(vel);
    note->setLength(llen);
    note->setStart(pos);
    note->setEnh(enh);
    note->setChan(chan);

    const char * orns[] = { "LYRICS", "EXP" };
    Element*(*funPtr[])(char*,ifstream*&,Element*) = { &Lyrics::load, &Expression::load };
    // Element::loadContent(note->ornament(),"/NOTE","EXP",&Expression::load,inPtr);
    
    Element::loadContent(note->ornament(),"/NOTE", 2, orns, funPtr, inPtr);

    attr->scratch();
    delete attr;
  }
  return note;
}


#endif
