#ifndef _NOTE_H_
#define _NOTE_H_

#include "event.h"
#include "table.h"
#include "expression.h"
#include "bow.h"

class Ornament;
class Vector;
class Lyrics;

enum {
  NS_STACCATO, NS_LEGATO, NS_ACCDOWN, NS_ACCUP, NS_ACCENT1, NS_ACCENT2, NS_ACCENT3, NS_ACCENT4,
  NS_ACCENT5, NS_ACCENT6, NS_ACCENT7, NS_ACCENT8, NS_ACCENT9, NS_ACCENT10, NS_ACCENT11, NS_TRILLER,
  NS_TRILL1, NS_TRILL2, NS_INF, NS_BOWUP, NS_BOWDOWN };

enum {
  AS_LABEL1, AS_LABEL2, AS_LABEL3, AS_CASE1, AS_CASE2, AS_PEDAL1, AS_PEDAL2, AS_NOTEDOT,
  AS_NOTE3, AS_NOTE6, AS_NOTEXX, AS_8VA, AS_15VA, AS_TRILLX, AS_TRILLX2, AS_DECRESCENDO,
  AS_CRESCENDO, AS_BRACKETUP, AS_BRACKETDOWN, AS_ARPEGGIO, AS_TEXT,
  DYN_PPP, DYN_PP, DYN_P, DYN_MP, DYN_MF, DYN_F,
  DYN_FF, DYN_FFF, DYN_SFZ, DYN_SF, DYN_SFF, DYN_FP };

/** The note event contains a pitch, velocity, enharmonic shift, channel and a vector of ornaments. */
class Note : public Event
{
 private:
  int _pitch;
  int _vel;
  int _enh;
  int _chan;
  Vector * _ornament;
  char * _cPitch;

 public:
  /** Default constructor */
  Note( );

  /** Constructor used by copy() */
  Note(Note*);

  /** Use this constructor to build a new note */
  Note(int pitch, int vel, long len, Position pos, int enh, int c=-1);

  /** Use this constructor to build a new note */
  Note(char* pitch, int vel, long len, Position pos, int enh, int c=-1);

  /** This destructor gets rid of the ornament vector */
  ~Note();
  // virtual bool operator==(Element*);

  /** returns the pitch */
  int pitch();

  /** returns the velocity */
  int vel();
  
  /** returns the enharmonic shift */
  int enh();

  /** returns the channel. -1 invikes the default (part defined) channel */
  int chan();

  /** returns the ornament-vector */
  Vector * ornament();

  /** sets the pitch */
  void setPitch(int f);

  /** sets the velocity */
  void setVel(int v);

  /** sets the enharmonic shift */
  void setEnh(int e);

  /** sets the MIDI channel */
  void setChan(int c);

  /** adds an ornament to the ornament vector */
  void add(Ornament*);

  /** removes an ornament from the ornament vector.
      The ornament is not deleted here, deletion is done in AddOrnament's destructor
  */
  void remove(Ornament*);

  /** returns an ornament of the ornament vector, and creates it, if necessary */
  Expression * setExpression(int);

  /** returns an ornament of the ornament vector, and creates it, if necessary */
  Bow * setBow(int,int,int);

  /** returns the note's lyrics. No new lyrics are created if they don't exist yet. */
  Lyrics * lyrics();


  // void sExt(KbNoteExt*);
  // void appendExt(KbNoteExt*);
  // KbExp * giveExp(int);
  // KbBow * giveBow(int,int,int);
  // KbLyrics * lyrics();
  // int stemDir(); // stemDir = -1: undefined, 0: down, 1: up
  // int expression();
  // int bowLength();
  // KbExp * gExp(KbNoteExt*start=0);
  // KbBow * gBow();
  // void flipStem();
  // void flipBow();

  void removeExp();

  void removeBow();

  /** returns the pitch in a nice character presentation */
  char * cPitch();

  /** implementation of the print method */
  virtual ostream & print(int,ostream&);

  /** implementation of the flush method */
  virtual void flush(char*);

  /** implementation of the copy method */
  virtual Element * copy();

  /** implementation of the load method */
  static Element * load(char*,ifstream*&,Element*);

};


#endif
