#ifndef _PLAYER_CPP_
#define _PLAYER_CPP_

#include <iostream.h>
#include <string.h>
#include <libkmid/midiout.h>

#include "player.h"
#include "reference.h"
#include "part.h"
#include "track.h"
#include "note.h"
#include "midiEvent.h"
#include "song.h"
#include "table.h"
#include "element.h"
#include "scoreTrack.h"
#include "drumTrack.h"
#include "prProgress.h"

Player::Player() : _song(0), devnum(0), playing(false) {
  init();
}

Player::Player(Song * s) : _song(s), devnum(0), playing(false) {
  init();
}

Player::~Player() {
  // TODO: delete devman; // causes a segfault invalid filedescriptor
}



void Player::init() {
  char * map_path = NULL;
  devman = new DeviceManager();
  devman->initManager();
  MidiMapper *map=new MidiMapper(map_path);
  devman->setMidiMap(map);
  char * line;
  devnum = devman->synthDevices() + devman->midiPorts(); // my AWE has 2 synths and 1 MIDI
  for (int i=0; i<devnum && i<MAXDEVNUM; i++) { devices[i] = strdup(devman->name(i)); line = devices[i]; line+=9; line[0]=0; }
}

char * Player::device(int i) { return devices[i]; }

int Player::devicenum() { return devnum; }

void Player::play(PrProgress * progress, Part * part, long left, long right) {
  if (!playing) {
    playing = true;

    int ntracks = 0;
    double progStep = 0;
    double progInd = 0;
    int progInd1 = 0;
    bool veto = false;
    
    if (right==0) progress = 0; // DOIT: calculate progStep for unspecified left+right values!
    else {
      progStep = 100.0/(right-left);
      if (progress!=0) progress->progress(0);
    }
    if (part!=0) {
      ntracks = 1;
    } else {
      ntracks = _song->size();
    }
    Track * tr[ntracks];
    Part * pt[ntracks];
    int stp[ntracks];
    
    if (part!=0) {
      pt[0] = part;
      // pt[0]->rewind();
      pt[0]->wind(left);
      tr[0] = 0;
      stp[0] = 0;
    } else {
      for (int t=0; t<ntracks; t++) {
	tr[t] = (Track*) _song->get(t);
	pt[t] = (Part*) tr[t]->first();
	// pt[t]->rewind();
	if (pt[t]) {
	  pt[t]->wind(left);
	  stp[t] = 0;
	  // if (tr[t]->isA()==SCORETRACK) devman->setDevice(((ScoreTrack*)tr[t])->channel(),((ScoreTrack*)tr[t])->output());
	  // if (tr[t]->isA()==DRUMTRACK) devman->setDevice(((DrumTrack*)tr[t])->channel(),((DrumTrack*)tr[t])->output());
	  if (tr[t]->isA()==SCORETRACK) {
	    if (((ScoreTrack*)tr[t])->output() < devnum)
	      devman->chn2dev[((ScoreTrack*)tr[t])->channel()] = ((ScoreTrack*)tr[t])->output();
	    else veto = true;
	  }
	  if (tr[t]->isA()==DRUMTRACK) {
	    if (((DrumTrack*)tr[t])->output() < devnum)
	      devman->chn2dev[((DrumTrack*)tr[t])->channel()] = ((DrumTrack*)tr[t])->output();
	    else veto = true;
	  }
	}
      }
    }
    if (!veto) {
      devman->openDev();
      devman->initDev();
      devman->tmrSetTempo(100);
      devman->setVolumePercentage(100);
      // devman->tmrStart();
      // cout << "start playing" << endl;
      
      for (int t=0; t<ntracks; t++) {
	if (tr[t]!=0) {
	  if (tr[t]->isA()==SCORETRACK) devman->chnPatchChange(((ScoreTrack*)tr[t])->channel(),((ScoreTrack*)tr[t])->program());
	  // cout << ((ScoreTrack*)tr[t])->channel() << " -> " << ((ScoreTrack*)tr[t])->program() << endl;
	  if (tr[t]->isA()==DRUMTRACK) devman->chnPatchChange(((DrumTrack*)tr[t])->channel(),((DrumTrack*)tr[t])->program());
	}
      }
      int stop = 0;
      long pp = left;
      long oldTimePos = 0;
      needSync = false;
      Note * note;
      MidiEvent * mev;
      Element * el = 0;
      // Table * sus = new Table();
      Reference * ref = 0;
      double tempo = _song->tempo()*0.01;
      
      while (stop<2) {
	if (needSync) { devman->sync(); needSync = false; }
	
	for (int t=0; t<ntracks; t++) {
	  if (pt[t]) {
	    for (pt[t]->startMem();pt[t]->moreMem();) {
	      if (pt[t]->memEndsAt(pp)) {
		note = (Note*) pt[t]->mem();
		wait(pp*tempo,left*tempo);
		devman->noteOff(0,note->pitch(),note->vel());
		// cout << "stop " << note << endl;
		pt[t]->cutMem();
	      } else {
		pt[t]->incMem();
	      }
	    }
	  }
	}
	
	for (int t=0; t<ntracks; t++) {
	  if (pt[t]!=0) {
	    while (pt[t]->currentStartsAt(pp)) {
	      if (pt[t]->current(NOTE)) {
		note = (Note*) pt[t]->current();
		wait(pp*tempo,left*tempo);
		devman->noteOn(0,note->pitch(),note->vel());
		// cout << "start " << note << endl;
		pt[t]->rememberCurrent();
	      } else if (pt[t]->current(MIDIEVENT)) {
		mev = (MidiEvent*) pt[t]->current();
		wait(pp*tempo,left*tempo);
		switch (mev->code()) {
		case 10: devman->keyPressure(mev->channel(), mev->value1(), mev->value2()); break; // Aftertouch, Poly Pressure
		case 11: devman->chnController(mev->channel(), mev->value1(), mev->value2()); break; // Control Change
		case 12: devman->chnPatchChange(mev->channel(), mev->value1()); break; // Program Change
		case 13: devman->chnPressure(mev->channel(), mev->value1()); break; // Aftertouch, Channel Pressure
		case 14: devman->chnPitchBender(mev->channel(), mev->value1(), mev->value2()); break; // Pitchbend
		case 15: cout << "sysex msg" << endl; break;
		}
	      } else {
		cout << "track " << t << *pt[t]->current() << endl;
	      }
	      if (!pt[t]->currentInc()) stp[t]++;
	    }
	  }
	}
	stop = 1;
	if ((right!=0) && (pp>=right)) {
	} else {
	  for (int t=0; t<ntracks; t++) { if (stp[t]==0) stop = 0; }
	}
	
	if (stop==1) {
	  stop = 2;
	  for (int t=0; t<ntracks; t++) {
	    pt[t]->startMem();
	    if (pt[t]->moreMem()) stop = 1;
	  }
	}
	pp++;
	if (progress!=0) { progInd += progStep; if (int(progInd) > progInd1) { progress->progress(int(progInd)); progInd1 = int(progInd); } }
      }
      
      pp += 384;
      wait(pp*tempo,left*tempo);
      devman->sync();
      // cout << "stop playing" << endl;
      devman->tmrStop();
      devman->closeDev();
      
      for (int t=0; t<ntracks; t++) {
	pt[t]->startMem();
	if (pt[t]->moreMem()) {
	  pt[t]->clearMem();
	}
      }
    }
    playing = false;
  }
  // cout << sus->length() << endl;
  // delete sus;
}


void Player::wait(long timePos, long offset) {
  if (oldTimePos!=timePos) {
    devman->wait(timePos-offset);
    oldTimePos=timePos;
  }
  needSync=true;
}

void Player::hit(int out, int ch, int freq, int vol) {
  if (!playing) {
    playing = true;
    if (devnum>out) {
      devman->chn2dev[ch] = out;
      devman->openDev();
      devman->initDev();
      if (freq!=0) {
	devman->tmrSetTempo(100);
	devman->setVolumePercentage(100);
	// devman->chnPatchChange(ch,0);
	devman->wait(0);
        devman->noteOn(ch,freq,vol);
	devman->sync();
        devman->wait(200);
        devman->noteOff(ch,freq,vol);
        devman->sync();
        devman->tmrStop();
      }
      devman->closeDev();
    }
    playing = false;
  }
}

void Player::quad(int out, int ch, int freq1, int freq2, int freq3, int freq4, int vol) {
  if (!playing) {
    playing = true;
    if (devnum>out) {
      devman->chn2dev[ch] = out;
      devman->openDev();
      devman->initDev();
      devman->tmrSetTempo(100);
      devman->setVolumePercentage(100);
      //devman->chnPatchChange(ch,0);
      devman->wait(0);
      if (freq1!=0) devman->noteOn(ch,freq1,vol);
      if (freq2!=0) devman->noteOn(ch,freq2,vol);
      if (freq3!=0) devman->noteOn(ch,freq3,vol);
      if (freq4!=0) devman->noteOn(ch,freq4,vol);
      devman->sync();
      devman->wait(600);
      if (freq1!=0) devman->noteOff(ch,freq1,vol);
      if (freq2!=0) devman->noteOff(ch,freq2,vol);
      if (freq3!=0) devman->noteOff(ch,freq3,vol);
      if (freq4!=0) devman->noteOff(ch,freq4,vol);
      devman->sync();
      devman->tmrStop();
    }
    devman->closeDev();
    playing = false;
  }
}


#endif
