#ifndef _PRMAINEDITOR_CPP_
#define _PRMAINEDITOR_CPP_

#include <dlfcn.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include "prMainEditor.h"
#include "prPartEditor.h"
#include "addon.h"
#include "song.h"
#include "loader.h"

int generalAddonsSize = 3;
char * generalAddons[] = { "quantizeAll", "quantizeLength", "fixedLength" };

extern Song * sonG;
extern char * parameter(const char * name, const char * alt, int argc, char ** argv);


char ** addons(const char * name, const char * alt, int argc, char ** argv, int & count) {
  char ** ret = 0;
  count = 0;
  for (int i=0; i<argc-1; i++) if ((strcmp(argv[i],name)==0)||(strcmp(argv[i],alt)==0)) count++;
  count += generalAddonsSize;
  ret = new char*[count];
  int c = 0;
  for (int i=0; i<argc-1; i++) {
    if ((strcmp(argv[i],name)==0)||(strcmp(argv[i],alt)==0)) ret[c++] = strdup(argv[i+1]);
  }
  for (int i=0; i<generalAddonsSize; i++) {
    ret[c++] = generalAddons[i];
  }
  return ret;
}


PrMainEditor::PrMainEditor(int argc, char** argv) {
  for (int i=0;i<MAXACTIONS;i++) _action_list[i] = 0;
  for (int i=0;i<MAXEDITORS;i++) _editor_list[i] = 0;
  
  _print_in_partiture = true;
  _print_in_piano_system = false;
  _file_name = strdup("untitled.bms");
  _undo_memory = 20;
  _grid = 384;

  //
  // Addons:
  // -------
  //
  
  char * DLERROR;
  
  int count = 0;
  char ** addonList = addons("--addon", "-a",argc,argv,count);
  _addons = new OpFunction[count];

  //
  // loop through the addon parameters:
  //
  for (int i=0; i<count; i++) {
    char * name = addonList[i];
    char c = name[0]; // capitalize first letter, (so "dump" -> "Dump")
    if (c > 96) c -= 32;
    char * addon = new char[50];
    char * opdesc = new char[50];
    char * opctgr = new char[50];
    char * opcont = new char[50];
    name++;
    sprintf(addon,"libBrahmsAddon%c%s.so",c,name);
    name--;
    sprintf(opdesc,"%s_name",name);
    sprintf(opctgr,"%s_category",name);
    sprintf(opcont,"%s_context",name);
#ifdef DEBUGIT
    cout << "trying to open addon " << i << ": " << addon << " ..." << endl;
#endif
    void * handleAddon = dlopen (addon, RTLD_LAZY | RTLD_GLOBAL );
    if (!handleAddon) {
      cout << "error open addon: " << dlerror() << endl;
      // exit(1);
    } else {
  
#ifdef DEBUGIT
      cout << "loading symbol " << name << " ..." << endl;
#endif
      OpFunction getAddon = (OpFunction) dlsym(handleAddon,name);
      if ((DLERROR = dlerror()) != NULL)  cout << "error sym: " << DLERROR << endl;
      else {
#ifdef DEBUGIT
	cout << "loading symbol " << opdesc << " ..." << endl;
#endif
	OpDescriptor description = (OpDescriptor) dlsym(handleAddon,opdesc);
	if ((DLERROR = dlerror()) != NULL)  cout << "error sym: " << DLERROR << endl;
	else {
#ifdef DEBUGIT
	  cout << "loading symbol " << opctgr << " ..." << endl;
#endif
	  OpCategory category = (OpCategory) dlsym(handleAddon,opctgr);
	  if ((DLERROR = dlerror()) != NULL)  cout << "error sym: " << DLERROR << endl;
	  else {
#ifdef DEBUGIT
	    cout << "loading symbol " << opcont << " ..." << endl;
#endif
	    OpContext context = (OpContext) dlsym(handleAddon,opcont);
	    if ((DLERROR = dlerror()) != NULL)  cout << "error sym: " << DLERROR << endl;
	    else {
	      _addons[i] = getAddon;
	      _action_list[i] = description();
	      _action_category[i] = category();
	      _action_context[i] = context();
	    }
	  }
	}
      }
    }
  }
}

int PrMainEditor::grid() { return _grid; }

//int PrMainEditor::gridx() { return _grid; cout << "takex " << _grid << endl; }

void PrMainEditor::setGrid(int i) { _grid = i; }

void PrMainEditor::loadSong(int argc, char ** argv) {
  //
  // Load Song:
  // ----------
  //

  char * fname = parameter("--file","-f",argc,argv);

  if (fname==0) {
    sonG = new Song();
  } else {
    sonG = Loader::load(fname);
  }
  
}

char ** PrMainEditor::actionList() { return _action_list; }

char * PrMainEditor::actionCategory(int i) { return _action_category[i]; }

int PrMainEditor::actionContext(int i) { return _action_context[i]; }

void PrMainEditor::performAction(int i, Element * target) {
  Addon * op =  (_addons)[i](target);
  sonG->doo(op);
  update();
}

char ** PrMainEditor::categories() {
  char ** ret = 0;
  ret = new char*[MAXACTIONS];
  int c = 0;
  for (int i=0; i<MAXACTIONS; i++) {
    if (_action_list[i]!=0) {
      bool newcat = true;
      for (int k=0; k<c; k++) if (strcmp(_action_category[i],ret[k])==0) newcat = false;
      if (newcat) ret[c++] = strdup(_action_category[i]);
    }
  }
  for (int i=c; i<MAXACTIONS; i++) ret[i] = 0;
  return ret;
}

int * PrMainEditor::actionListByCategory(char* cat) {
  int * ret = new int[MAXACTIONS];
  int c = 0;
  for (int i=0; i<MAXACTIONS; i++) {
    if (_action_list[i]!=0 && strcmp(_action_category[i],cat)==0) ret[c++] = i;
  }
  for (int i=c; i<MAXACTIONS; i++) ret[i] = -1;
  // for (int i=0; ret[i]!=-1; i++) { cout << i << ": " << ret[i] << endl; }
  return ret;
}




void PrMainEditor::addEditor(PrPartEditor * ed) {
  int i=0;
  for (i=0; _editor_list[i]!=0; i++) {}
  if (i<MAXEDITORS) _editor_list[i] = ed;
  else {
    cout << "PANIC: more than " << MAXEDITORS << " editors opened" << endl;
    delete ed;
  }
}

void PrMainEditor::removeEditor(PrPartEditor * ed) {
  int ind = -1;
  for (int i=0; _editor_list[i]!=0; i++)
    if (_editor_list[i] == ed) ind = i;
  if (ind!=-1) {
    for (int i = ind; _editor_list[i]!=0; i++)
      _editor_list[i] = _editor_list[i+1];
  }
}

void PrMainEditor::dumpEditors() {
  cout << "\nOpen editors:\n" << endl;
  for (int i=0; _editor_list[i]!=0; i++)
    cout << i << ": " << _editor_list[i]->name() << endl;
}

void PrMainEditor::ui(GuiEvents ev, bool flag) {
  gui(ev,flag);
  // update(); // slows down things a little bit!
  for (int i=0; _editor_list[i]!=0; i++) {
    _editor_list[i]->gui(ev,flag);
    _editor_list[i]->update();
  }
}

char * PrMainEditor::filename() { return _file_name; }

void PrMainEditor::setFilename(char * str) { _file_name = str; }

int PrMainEditor::undoMemory() { return _undo_memory; }

void PrMainEditor::setUndoMemory(int i) { _undo_memory = i; }


/* printing */
bool PrMainEditor::PP() { return _print_in_partiture; }

void PrMainEditor::sPP(bool p) { _print_in_partiture = p; }

bool PrMainEditor::PPS() { return _print_in_piano_system; }

void PrMainEditor::sPPS(bool p) { _print_in_piano_system = p; }

#endif
