#ifndef _PRMAINEDITOR_H_
#define _PRMAINEDITOR_H_

#define MAXACTIONS 80
#define MAXEDITORS 20

#include "element.h"

class Addon;
class Element;
class PrPartEditor;

typedef Addon * (*OpFunction)(Element*);
typedef char  * (*OpDescriptor)();
typedef char  * (*OpCategory)();
typedef int     (*OpContext)();

class PrMainEditor
{
 private:
  OpFunction *   _addons;
  char *   _action_list[MAXACTIONS];
  char *   _action_category[MAXACTIONS];
  int            _action_context[MAXACTIONS];
  PrPartEditor * _editor_list[MAXEDITORS];
  int            _undo_memory;
  bool           _print_in_partiture;
  bool           _print_in_piano_system;
  int            _grid;

  // protected:
  char *       _file_name;

 public:
  PrMainEditor(int,char**);
  char ** actionList();
  char * actionCategory(int);
  int actionContext(int);
  void performAction(int,Element*);
  char ** categories();
  int * actionListByCategory(char*);

  void addEditor(PrPartEditor*);
  void removeEditor(PrPartEditor*);
  void dumpEditors();
  void ui(GuiEvents,bool);
  void loadSong(int,char**);
  char * filename();
  void setFilename(char*);
  int undoMemory();
  void setUndoMemory(int);
  int grid();
  void setGrid(int);

  virtual void run()=0;
  virtual void update()=0;
  virtual void gui(GuiEvents,bool)=0;

  /* printing */

  /** Print-in-Partiture: returns true if this partiture printing is enabled */
  bool PP();

  /** sets the Print-in-Partiture flag */
  void sPP(bool p);

  /** Print-in-Piano-System: returns true if piano system printing is enabled  */
  bool PPS();

  /** sets the Print-in-Piano-System flag */
  void sPPS(bool p);

};

#endif
