#ifndef _PRPARTEDITOR_CPP_
#define _PRPARTEDITOR_CPP_

#include <stdio.h>
#include <iostream.h>
#include <math.h>

#include "prMainEditor.h"
#include "prPartEditor.h"
#include "table.h"
#include "note.h"
#include "part.h"
#include "track.h"
#include "reference.h"
#include "str.h"
#include "song.h"
#include "event.h"

extern Song * sonG;
extern Table * selectioN;
extern PrMainEditor * mainEditor;

PrPartEditor::PrPartEditor() : _parts(new Table()), _current(0), _leftPos(0), _bars(4), _rightPos(0),
			       _cLeft(new char [13]), _cRight(new char[13]), _current_length(192), _grid(192),
			       _current_enh(0), _current_vel(87), _current_dot(0), _current_tri(false), _speaker(true), _color(false) {
  // no parameter means: edit song:
  char * name = 0;
  int imax = sonG->size();
  Track * tr = 0;
  Part * pt = 0;
  for (int i=0; i<imax; i++) {
    name = 0;
    tr = (Track*) sonG->get(i);
    if ( (tr != 0) && (!tr->mute()) ) {
      String * s = tr->name();
      if (s) name = s->getValue();
      if (name==0) { name = new char[2]; name[0] = '-'; name[1] = 0; }
      _parts->add(new Reference(name,tr->first()));
    }
  }
  _current = (Reference*) _parts->first();
  if (_current) wind(((Part*)_current->getValue())->start());
}

PrPartEditor::PrPartEditor(Part * p) : _parts(new Table()), _current(0), _leftPos(0), _bars(4), _rightPos(0),
				       _cLeft(new char [13]), _cRight(new char[13]), _current_length(192), _grid(192),
				       _current_enh(0), _current_vel(87), _current_dot(0), _current_tri(false), _speaker(true), _color(false) {
  _current = new Reference((char*)p->ctype(),p);
  _parts->add(_current);
  wind(p->start());
}

PrPartEditor::PrPartEditor(Track * t) : _parts(new Table()), _current(0), _leftPos(0), _bars(4), _rightPos(0),
					_cLeft(new char [13]), _cRight(new char[13]), _current_length(192), _grid(192),
					_current_enh(0), _current_vel(87), _current_dot(0), _current_tri(false), _speaker(true), _color(false) {

  char * name = 0;
  Part * p = 0;
  if (t!=0) {
    p = (Part*) t->first(); // TODO: What to do with the following parts?
    String * s = t->name();
    if (s) name = s->getValue();
  }
  if (name==0) { name = new char[2]; name[0] = '-'; name[1] = 0; }
  if (p!=0) {
    _current = new Reference(name,p);
    _parts->add(_current);
  }
  wind(((Part*)_current->getValue())->start());
  cout << "TODO: set flag to use all parts in track or so..." << endl;
}

PrPartEditor::~PrPartEditor() {
  mainEditor->removeEditor(this);
  mainEditor->update();
  if (_parts!=0)
    delete _parts;
  delete _cLeft;
  delete _cRight;
}

Part * PrPartEditor::part() {
  Part * pt = 0;
  if (_current) pt = (Part*) _current->getValue();
  return pt;
}

Part * PrPartEditor::part(int i) {
  Part * pt = 0;
  Reference * ref = (Reference*) _parts->get(i);
  if (ref) pt = (Part*) ref->getValue();
  return pt;
}

Part * PrPartEditor::setPart(int i) {
  Reference * ref = _current;
  _current = (Reference*) _parts->get(i);
  if (_current==0) _current = ref;
  return part();
}

int PrPartEditor::parts() { return _parts->size(); }


char * PrPartEditor::name() {
  char * rv = 0;
  if (_current) rv = _current->getKey();
  return rv;
}

Part * PrPartEditor::moveDown() {
  _current = (Reference*) _parts->next(_current);
  if (_current==0) _current = (Reference*) _parts->first();
  return (Part*) _current->getValue();
}

Part * PrPartEditor::moveUp() {
  _current = (Reference*) _parts->prev(_current);
  if (_current==0) _current = (Reference*) _parts->last();
  return (Part*) _current->getValue();
}


char * PrPartEditor::cLeft() {
  // format: bbb.bb.ttt
  int bar; int beat; int tick;
  sonG->bbt(bar,beat,tick,_leftPos);
  sprintf(_cLeft,"%3d.%2d.%3d",bar,beat,tick);
  return _cLeft;
}

char * PrPartEditor::cRight() {
  // format: bbb.bb.ttt
  int bar; int beat; int tick;
  sonG->bbt(bar,beat,tick,_rightPos);
  sprintf(_cRight,"%3d.%2d.%3d",bar,beat,tick);
  return _cRight;
}

Position PrPartEditor::left() {
  return _leftPos;
}

Position PrPartEditor::right() {
  return _rightPos;
}

void PrPartEditor::moveleft(int i) {
  for (int k=0; k<i; k++) {
    _leftPos.nextBar(sonG->master(), sonG->meter0(), sonG->meter1());
    _rightPos.nextBar(sonG->master(), sonG->meter0(), sonG->meter1());
  }
  update();
}

void PrPartEditor::moveright(int i) {
  for (int k=0; k<i; k++) {
    _leftPos.prevBar(sonG->master(), sonG->meter0(), sonG->meter1());
    _rightPos.prevBar(sonG->master(), sonG->meter0(), sonG->meter1());
  }
  update();
}

void PrPartEditor::rewind(Part * p) {
  /*  if (all==false) {
    if (part()!=0) part()->wind(_leftPos);
  } else {
    for (Reference * ref = (Reference*) _parts->first(); ref != 0; ref = (Reference *) _parts->next(ref)) {
      Part * pt = (Part*) ref->getValue();
      if (pt!=0) pt->wind(_leftPos);
    }
  } */
  Part * pt = p;
  if (pt==0) pt = part();
  if (pt!=0) pt->wind(_leftPos);
  _rightPos = _leftPos;
  for (int k=0; k<_bars; k++) {
    _rightPos.nextBar(sonG->master(), sonG->meter0(), sonG->meter1());
  }
}

void PrPartEditor::wind(int bar) {
  _leftPos.set(bar+1, 1, 0, sonG->master(), sonG->meter0(), sonG->meter1());
  if (part()!=0) part()->wind(_leftPos);
  _rightPos = _leftPos;
   for (int k=0; k<_bars; k++) {
    _rightPos.nextBar(sonG->master(), sonG->meter0(), sonG->meter1());
  }
}

void PrPartEditor::wind(Position p) {
  _leftPos = p;
  if (part()!=0) part()->wind(_leftPos);
  _rightPos = _leftPos;
  for (int k=0; k<_bars; k++) {
    _rightPos.nextBar(sonG->master(), sonG->meter0(), sonG->meter1());
  }
}

bool PrPartEditor::currentInc(Part * p, long ticks) {
  Part * pt = p;
  if (pt==0) pt = part();
  bool ok = true;
  if (ticks==0) { ticks = _rightPos.ticks(); }
  ok = pt->currentInc();
  if (ok)
    ok = pt->currentStartsBefore(ticks);
  // cout << "PrPartEditor::currentInc returns " << ok << endl;
  return ok;
}

void PrPartEditor::zoomin() {
  _bars--;
  if (_bars<1) {
    _bars = 1;
  } else {
    update();
  }
}

void PrPartEditor::zoomout() {
  _bars++;
  if (_bars>20) {
    _bars = 20;
  } else {
    update();
  }
}

int PrPartEditor::length() { return _current_length; }

int PrPartEditor::totalLength() {
  int ret = _current_length;
  for (int i=0; i<_current_dot; i++) ret = int(ret*1.5);
  return ret;
}

int PrPartEditor::lengthOrd() { return 10-int(1.000001*log(_current_length/3)/log(2)); }

void PrPartEditor::setLength(int l) { _current_length = l; }

void PrPartEditor::setLengthOrd(int l) { // 1536 = 1, 768 = 2, 384 = 3, 192 = 4, etc
  _current_length = int(3*pow(2,10-l));
}

int PrPartEditor::dot() { return _current_dot; }

void PrPartEditor::setDot(int d) { _current_dot = d; }

bool PrPartEditor::triplet() { return _current_tri; }

void PrPartEditor::setTriplet(bool t) { _current_tri = t; }


int PrPartEditor::enharmonicShift() { return _current_enh; }

void PrPartEditor::setEnharmonicShift(int e) { _current_enh = e; }

int PrPartEditor::velocity() { return _current_vel; }

void PrPartEditor::setVelocity(int v) { _current_vel = v; }


Note * PrPartEditor::getGrabNote(long mpos, int mpitch, bool & tail) {
  Note * n = 0;
  Note * ret = 0;
  bool ok = true;
  Part * _part = part();
  Event * event;
  int len = 0;
  long pos = 0;
  int pi = 0;
  _part->rewind();
  while (ok) {
    event = _part->current();
    if (event!=0) {
      pos = _part->currentPos();
      if (event->isA()==NOTE) {
	n = (Note*) event;
	len = n->length();
	pi = n->pitch();
	// check:
	if ((pos <= mpos) && (mpos < pos+len) && (mpitch==pi)) {
	  ret = n;
	  if (mpos > pos+len - 16) tail = true;
	}
      }
    }
    ok = currentInc();
  }
  return ret;
}

long PrPartEditor::snap(long pos) {
  return pos - (pos%_grid);
}

int PrPartEditor::grid() { return _grid; }

void PrPartEditor::setGrid(int i) { _grid = i; mainEditor->setGrid(i); }

Event * PrPartEditor::currentEvent() {
  if (selectioN->size()==1) return (Event*) ((Reference*)selectioN->first())->getValue();
  else                      return 0;
}

bool PrPartEditor::color() { return _color; }

bool PrPartEditor::speaker() { return _speaker; }

void PrPartEditor::setColor(bool c) { _color = c; }
  
void PrPartEditor::setSpeaker(bool s) { _speaker = s; }


#endif
