#ifndef _KDEEDITINFO_CPP_
#define _KDEEDITINFO_CPP_

#include <iostream.h>

#include <qpixmap.h>
#include <qpainter.h>
#include <qcursor.h>
#include <qcombobox.h>

#include "kdeEditInfo.h"

#include "../../prPartEditor.h"
#include "../../note.h"
#include "../../part.h"

enum { ID_VOLUME };

KdeEditInfo::KdeEditInfo( QWidget * parent, PrPartEditor * ed )
  : QFrame(parent,"info",QFrame::Raised), _editor(ed), _xoffset(80), _pressX(0), _pressY(0), _moveX(0), _moveY(0), _grabX(-1)
{
    setCursor(QCursor(2));
    
    QComboBox * ctrl = new QComboBox(FALSE,this,"control");
    ctrl->insertItem("vol", ID_VOLUME);
    ctrl->setGeometry(20,40,_xoffset-24,24);
    ctrl->show();
    ctrl->clearFocus();
}

void KdeEditInfo::paintEvent( QPaintEvent * pe ) {
  QPixmap pix( width(), height() );
  pix.fill( this, 10,10 );
  QPainter painter;
  painter.begin( &pix );
  
  int ht = height();
  scl = (ht-10.0)/128;

  Part * _part = _editor->part();
  _editor->rewind();
  int vel = 0;
  int xx = 0;
  int xxx = width()-20;
  Position pos;
  long _leftPos = _editor->left().ticks();
  long _rightPos = _editor->right().ticks();
  double pixPerTick = (xxx-_xoffset-4.0)/(_rightPos-_leftPos);

  bool ok = true;
  while (ok) {
    Event * event = _part->current();
    if (event!=0) {
      pos = _part->currentPos();
      if (event->isA()==NOTE) {
	Note * note = (Note*) event;
	vel = note->vel();
      }
      xx = long((pos-_leftPos)*pixPerTick + _xoffset);
      painter.drawRect(xx+1,ht-10,3,-int(vel*scl));

    }
    ok = _editor->currentInc();
  }
  
  if (_grabX > -1) {
    painter.setPen(Qt::DashLine);
    // if (_grab==0)
    painter.drawLine(_pressX,_pressY,_moveX,_moveY);
    painter.setPen(Qt::SolidLine);
  }

  painter.end();
  bitBlt( this, 0, 0, &pix );

}

void KdeEditInfo::mousePressEvent( QMouseEvent * me ) {
  _pressX = me->x();
  _pressY = me->y();
  double pixPerTick = (width()-_xoffset-24.0)/(_editor->right().ticks()-_editor->left().ticks());
  long pos = _editor->snap((_pressX-_xoffset)/pixPerTick) + _editor->left().ticks();
  // int pitch = Pitch(_pressY);
  _grabX = int((_pressX-_xoffset)/pixPerTick + _editor->left().ticks()); // without snap!

}

void KdeEditInfo::mouseMoveEvent( QMouseEvent * me ) {
  _moveX = me->x();
  _moveY = me->y();
  double pixPerTick = (width()-_xoffset-24.0)/(_editor->right().ticks()-_editor->left().ticks());
  long pos = _editor->snap((_moveX-_xoffset)/pixPerTick) + _editor->left().ticks();
  long mpos = long((_moveX-_xoffset)/pixPerTick) + _editor->left().ticks();
  repaint(FALSE);
}

void KdeEditInfo::mouseReleaseEvent( QMouseEvent * me ) {
  double pixPerTick = (width()-_xoffset-24.0)/(_editor->right().ticks()-_editor->left().ticks());
  long pos = _editor->snap((me->x()-_xoffset)/pixPerTick) + _editor->left().ticks();
  int value1 = 128 - int(_pressY/scl);
  int value2 = 128 - int(me->y()/scl);
  // int _grabY2 = mouseY();
  long _grabX2 = long((_moveX-_xoffset)/pixPerTick + _editor->left().ticks()); // without snap!
  // cout << _grabX << ", " << value1 << ", " << _grabX2 << ", " << value2 << endl;
  double coeff = (value2-value1)*1.0/(_grabX2-_grabX);
  Part * _part = _editor->part();
  if (!_part->wind(_grabX)) {
    cout << "PANIC: kdeEditInfo.cpp: cannot rewind part!" << endl;
  } else {
    bool ok = true;
    while (ok) {
      Event * event = _part->current();
      if (event!=0) {
	long x = event->start().ticks();
	if (x > _grabX2) ok = false;
	else if (event->isA()==NOTE) {
	  int newval = value1 + int(coeff*(x - _grabX));
	  ((Note*)event)->setVel(newval);
	}
      }
      ok = _editor->currentInc(_part);
    }
  }
  
  _grabX = -1;
  repaint(FALSE);

}


#endif
