#ifndef _KDEMAINEDITOR_CPP_
#define _KDEMAINEDITOR_CPP_

#include <stdlib.h>
#include <iostream.h>
#include <math.h>

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qspinbox.h>
#include <qprogressbar.h>
#include <qslider.h>
#include <qlineedit.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <ktmainwindow.h>
#include <kaccel.h>
#include <kiconloader.h>
#include <ktoolbar.h>

#include "../../song.h"
#include "../../scoreTrack.h"
#include "../../drumTrack.h"
#include "../../masterTrack.h"
#include "../../audioTrack.h"
#include "../../commentTrack.h"
#include "../../part.h"
#include "../../unselect.h"
#include "../../addElement.h"
#include "../../cutSelection.h"
#include "../../copySelection.h"
#include "../../pasteSelection.h"
#include "../../vector.h"
#include "../../reference.h"
#include "../../str.h"
#include "../../note.h"
#include "../../loader.h"
#include "../../addon.h"
#include "../../table.h"

#include "resources.h"
#include "kdeMainEditor.h"
#include "kdeArranger.h"
#include "kdeFactory.h"
#include "kdePartEditor.h"
#include "kdeScoreEditor.h"
#include "kdePianoRollEditor.h"
#include "kdeTrack.h"
#include "kdeProgress.h"
#include "kdeSpecialEffects.h"
#include "kdePrefs.h"
#include "headline.h"
#include "selectionFrame.h"
#include "timeBar.h"

#define MAXHEIGHT 2048

extern Song * sonG;
extern Table * selectioN;
extern Table * selectionMemory;


KdeMainEditor::KdeMainEditor(char * name, KApplication * app, int argc, char ** argv)
  : KTMainWindow(name), PrMainEditor(argc,argv), _app(app), _argc(argc), _argv(argv), _grid_id(ID_GRID_4),
    _track_width(632), _track_height(26), _pixPerTick(1.0/24), _bar_offset(0),
    _meter_text(new char[32]), _caption_text(new char[200]), _position_text(new char[32]), _tool(ID_PANEL_ARROW),
    _select_area(NONE) {

  initMenuBar();
  initStatusBar();
  initKeyAccel();

  initView();

  setCaption("Brahms");
  _right->setCursor(QCursor(arrTool,0,0));
  show();
}

void KdeMainEditor::run() {
  // initSong();
  loadSong(_argc,_argv);
  // for (Track * tr = (Track*) sonG->content(); tr != 0; tr = (Track*) sonG->next(tr)) tr->presentation()->update();
  update();
  gui(UNDO,false);
  gui(CHANGES,false);
  _app->exec();
}

void KdeMainEditor::update() {
  int i=0;
  for (Track * tr = (Track*) sonG->content(); tr != 0; tr = (Track*) sonG->next(tr)) {
    if (tr->presentation()!=0) tr->presentation()->update();
  }
  displaySongProperties();
  _timebar->repaint(FALSE);
}

SelectArea KdeMainEditor::selectArea() { return _select_area; }

void KdeMainEditor::selectArea(SelectArea a) {
  _select_area = a;
  if (a==NONE) sonG->doo(new Unselect());
}

void KdeMainEditor::displaySongProperties() {
  // Meter:
  snprintf(_meter_text,30," %d / %d",sonG->meter0(),sonG->meter1());
  _meterbox->setText(_meter_text);
  
  // Tempo:
  _tempobox->setValue(sonG->tempo());
  
  // Title:
  sprintf(_caption_text,"Brahms: %s (%s)", sonG->title(), filename());
  setCaption(_caption_text);
  
  // Position:
  int a = 0; int b = 0; int c = 0;
  sonG->bbt(a,b,c,sonG->current());
  snprintf(_position_text,30," %d. %d. %d",a,b,c);
  ((QLineEdit*) panel->getWidget(ID_PANEL_POS))->setText(_position_text);
  ((QLineEdit*) panel->getWidget(ID_PANEL_POS))->clearFocus();

}

int KdeMainEditor::snap() {
  int ret = 0;
  switch (_grid_id) {
  case ID_GRID_1: ret = 1536; break;
  case ID_GRID_2: ret = 768; break;
  case ID_GRID_4: ret = 384; break;
  case ID_GRID_8: ret = 192; break;
  case ID_GRID_16: ret = 96; break;
  case ID_GRID_32: ret = 48; break;
  case ID_GRID_64: ret = 24; break;
  case ID_GRID_OFF: ret = 0; break;
  }
  setGrid(ret);
  return ret;
}

void KdeMainEditor::gui(GuiEvents i, bool b) {
  switch (i) {
  case CHANGES:
    fileMenu->setItemEnabled( ID_FILE_SAVE, b );
    toolbar->setItemEnabled( ID_FILE_SAVE, b );
    break;
  case UNDO:
    editMenu->setItemEnabled( ID_EDIT_UNDO, b );
    toolbar->setItemEnabled( ID_EDIT_UNDO, b );
    break;
  case REDO:
    editMenu->setItemEnabled( ID_EDIT_REDO, b );
    toolbar->setItemEnabled( ID_EDIT_REDO, b );
    break;
  case SELECTIONS:
    toolbar->setItemEnabled( ID_EDIT_CUT, b );
    toolbar->setItemEnabled( ID_EDIT_COPY, b );
    toolbar->setItemEnabled( ID_EDIT_CLEAR, b );
    toolbar->setItemEnabled( ID_EDIT_TRANSPOSE, b );
    editMenu->setItemEnabled( ID_EDIT_CUT, b );
    editMenu->setItemEnabled( ID_EDIT_COPY, b );
    editMenu->setItemEnabled( ID_EDIT_CLEAR, b );
    editMenu->setItemEnabled( ID_EDIT_TRANSPOSE, b );
    editMenu->setItemEnabled( ID_EDIT_QUANTIZE_ALL, b );
    editMenu->setItemEnabled( ID_EDIT_QUANTIZE_LENGTH, b );
    editMenu->setItemEnabled( ID_EDIT_FIXED_LENGTH, b );
    break;
  case MEMORY:
    toolbar->setItemEnabled( ID_EDIT_PASTE, b );
    editMenu->setItemEnabled( ID_EDIT_PASTE, b );
    break;
  }
  update();
}


// ***********************************************************
// **                                                       **
// **                     Widgets                           **
// **                                                       **
// ***********************************************************
//

void KdeMainEditor::initView() {
  KIconLoader * loader = KGlobal::iconLoader();
  QPixmap qpLeft = loader->loadIcon( "Paper01", KIcon::Desktop );
  QPixmap qpRight = loader->loadIcon( "Planning-And-Probing-1", KIcon::Desktop );
  int wd = width();
  int ht = height();
  if (wd<680) wd = 680;

  setFixedWidth(wd);

  _scroller = new QScrollView( this, "Scroller", QFrame::Raised );
  _scroller->setHScrollBarMode(QScrollView::AlwaysOff);
  _scroller->setVScrollBarMode(QScrollView::AlwaysOn);
  _scroller->setGeometry(0,0,wd,MAXHEIGHT);
  
  _child = new QWidget(_scroller);
  _child->setGeometry(0,0,wd,MAXHEIGHT);
  _scroller->addChild(_child);
  
  _splitter = new QSplitter(_child);
  _splitter->setGeometry(0,0,wd,MAXHEIGHT);
  _splitter->setOpaqueResize(TRUE);
  
  
  _left = new QWidget( _splitter );
  _left->setFixedHeight(MAXHEIGHT);
  _left->setMaximumWidth(wd);
  _left->setMinimumWidth(12);
  _left->setBackgroundPixmap( qpLeft );
  
  QPopupMenu * userDef = new QPopupMenu();
  /*char ** alist = actionList();
    for (int i=0;i<MAXACTIONS;i++) if ((alist[i]!=0) && (actionContext(i)&Addon::TREAT_SONG)) userDef->insertItem(alist[i],i);*/
  char ** alist = actionList();
  char ** clist = categories();
  for (int i=0;clist[i]!=0;i++) {
    QPopupMenu * catMenu = new QPopupMenu();
    userDef->insertItem(clist[i],catMenu,i);
    int * nlist = actionListByCategory(clist[i]);
    for (int k=0;nlist[k]!=-1;k++) {
      if (actionContext(nlist[k])&Addon::TREAT_SONG) catMenu->insertItem(alist[nlist[k]],nlist[k]);
    }
    connect(catMenu,SIGNAL(activated(int)),SLOT(slotSongAddon(int)));
  }

  
  _right = new KdeArranger( _splitter, userDef );
  _right->setFixedHeight(MAXHEIGHT);
  _right->setMinimumWidth(wd-_track_width);
  _right->setMaximumWidth(wd);
  _right->setBackgroundPixmap( qpRight );


  _headline = new HeadLine(_track_height,_left);
  _headline->setFrameStyle( QFrame::Panel | QFrame::Raised );
  _headline->setGeometry(0,0,_track_width+1,_track_height);
  _headline->show();

  _timebar = new TimeBar(_right);
  _timebar->setFrameStyle( QFrame::Panel | QFrame::Raised );
  _timebar->setGeometry(0,0,1600,24);
  _timebar->setLineWidth(2);
  _timebar->show();

  
  setView( _scroller );
  show();
  setFixedHeight(ht);
}

int KdeMainEditor::tool() { return _tool; }

void KdeMainEditor::resizeEvent( QResizeEvent * re ) {
  KTMainWindow::resizeEvent(re);
  int wd = width();
  _child->setGeometry(0,0,wd,MAXHEIGHT);
  _splitter->setGeometry(0,0,wd,MAXHEIGHT);
  _left->setMaximumWidth(wd);
  _right->setMinimumWidth(wd-_track_width);
  _right->setMaximumWidth(wd);
  _scrollbar->setGeometry(0,2,wd,16);
}

void KdeMainEditor::initKeyAccel()
{
  keyAccel = new KAccel(this);
	
  // fileMenu accelerators
  keyAccel->connectItem(KStdAccel::New, this, SLOT(slotFileNew()));
  keyAccel->connectItem(KStdAccel::Open, this, SLOT(slotFileOpen()));
  keyAccel->connectItem(KStdAccel::Save, this, SLOT(slotFileSave()));
  keyAccel->connectItem(KStdAccel::Close, this, SLOT(slotFileClose()));
  keyAccel->connectItem(KStdAccel::Quit, this, SLOT(slotFileQuit()));
  // editMenu accelerators
  keyAccel->connectItem(KStdAccel::Undo, this, SLOT(slotEditUndo()));
  keyAccel->connectItem(KStdAccel::Redo, this, SLOT(slotEditRedo()));
  keyAccel->connectItem(KStdAccel::Cut, this, SLOT(slotEditCut()));
  keyAccel->connectItem(KStdAccel::Copy, this, SLOT(slotEditCopy()));
  keyAccel->connectItem(KStdAccel::Paste, this, SLOT(slotEditPaste()));
  //keyAccel->connectItem("E", this, SLOT(slotEditScore()));
  //keyAccel->connectItem("R", this, SLOT(slotEditPianoRoll()));

  keyAccel->connectItem(KStdAccel::Help, this, SLOT(appHelpActivated()));
			
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_NEW, KStdAccel::New);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_OPEN, KStdAccel::Open);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_SAVE, KStdAccel::Save);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_CLOSE, KStdAccel::Close);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_QUIT, KStdAccel::Quit);

  keyAccel->changeMenuAccel(editMenu, ID_EDIT_UNDO, KStdAccel::Undo);
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_REDO, KStdAccel::Redo);
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_CUT, KStdAccel::Cut);
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_COPY, KStdAccel::Copy);
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_PASTE, KStdAccel::Paste);
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_EDIT_SCORE, i18n("Edit Score"));
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_EDIT_PIANOROLL, i18n("Edit PianoRoll"));

  keyAccel->readSettings();
}

void KdeMainEditor::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR
  recentFilesMenu = new QPopupMenu();
  connect(recentFilesMenu, SIGNAL(activated(int)), SLOT(slotFileOpenRecent(int)));


  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu = new QPopupMenu();
  fileMenu->insertItem(BarIcon("filenew"), i18n("&New"), ID_FILE_NEW);
  fileMenu->insertItem(BarIcon("fileopen"), i18n("&Open..."), ID_FILE_OPEN);
  fileMenu->insertItem(i18n("Open &recent"), recentFilesMenu, ID_FILE_OPEN_RECENT);
  fileMenu->insertItem(i18n("&Close"), ID_FILE_CLOSE);
  fileMenu->insertSeparator();
  fileMenu->insertItem(BarIcon("filesave") ,i18n("&Save"), ID_FILE_SAVE);
  fileMenu->insertItem(i18n("Save &As..."), ID_FILE_SAVE_AS);
  fileMenu->insertSeparator();
  fileMenu->insertItem( i18n("Import MIDI..."), ID_FILE_IMPORT_MIDI);
  fileMenu->insertItem( i18n("Export MIDI..."), ID_FILE_EXPORT_MIDI);
  fileMenu->insertSeparator();
  fileMenu->insertItem( i18n("Append..."), ID_FILE_APPEND);
  fileMenu->insertItem( i18n("Append MIDI..."), ID_FILE_APPEND_MIDI);
  fileMenu->insertSeparator();
  fileMenu->insertItem( i18n("Special effects"), ID_FILE_SPECIAL_EFFECTS);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("E&xit"), ID_FILE_QUIT);
  
  fileMenu->setItemEnabled( ID_FILE_SAVE, FALSE );

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu
  editMenu = new QPopupMenu();

  editMenu->insertItem(BarIcon("undo"), i18n("Undo"), ID_EDIT_UNDO);
  editMenu->insertItem(BarIcon("redo"), i18n("Redo"), ID_EDIT_REDO);
  editMenu->insertItem(BarIcon("eraser"), i18n("Clear History"), ID_EDIT_CLEAR_UNDO_HISTORY);
  editMenu->insertSeparator();
  editMenu->insertItem( i18n("Add scoretrack"), ID_EDIT_ADD_SCORETRACK);
  editMenu->insertItem( i18n("Add drumtrack"), ID_EDIT_ADD_DRUMTRACK);
  editMenu->insertItem( i18n("Add mastertrack"), ID_EDIT_ADD_MASTERTRACK);
  editMenu->insertItem( i18n("Add audiotrack"), ID_EDIT_ADD_AUDIOTRACK);
  editMenu->insertItem( i18n("Add comment"), ID_EDIT_ADD_COMMENT);
  editMenu->insertSeparator();
  editMenu->insertItem(BarIcon("editcut"), i18n("Cu&t"), ID_EDIT_CUT);
  editMenu->insertItem(BarIcon("editcopy"), i18n("&Copy"), ID_EDIT_COPY);
  editMenu->insertItem(BarIcon("editpaste"), i18n("&Paste"), ID_EDIT_PASTE);
  editMenu->insertSeparator();
  editMenu->insertItem( i18n("Edit score"), ID_EDIT_EDIT_SCORE);
  editMenu->insertItem( i18n("Edit pianoroll"), ID_EDIT_EDIT_PIANOROLL);
  editMenu->insertSeparator();
  editMenu->insertItem( i18n("Mixdown"), ID_EDIT_MIXDOWN);
  editMenu->insertSeparator();
  editMenu->insertItem( i18n("Mark left"), ID_EDIT_MARK_LEFT);
  editMenu->insertItem( i18n("Mark right"), ID_EDIT_MARK_RIGHT);
  editMenu->insertSeparator();
  /*editMenu->insertItem( i18n("Quantize all"), ID_EDIT_QUANTIZE_ALL);
  editMenu->insertItem( i18n("Quantize lengths"), ID_EDIT_QUANTIZE_LENGTH);
  editMenu->insertItem( i18n("Fixed lengths"), ID_EDIT_FIXED_LENGTH);
  editMenu->insertSeparator();*/
  editMenu->insertItem( i18n("Preferences"), ID_EDIT_PREFERENCES);
  
  editMenu->setItemEnabled( ID_EDIT_UNDO, FALSE );
  editMenu->setItemEnabled( ID_EDIT_REDO, FALSE );
  editMenu->setItemEnabled( ID_EDIT_CUT, FALSE );
  editMenu->setItemEnabled( ID_EDIT_COPY, FALSE );
  editMenu->setItemEnabled( ID_EDIT_PASTE, FALSE );
  
  editMenu->setAccel(ALT+Key_E,ID_EDIT_EDIT_SCORE);
  editMenu->setAccel(ALT+Key_R,ID_EDIT_EDIT_PIANOROLL);
  editMenu->setAccel(ALT+Key_H,ID_EDIT_CLEAR_UNDO_HISTORY);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry gridMenu
  gridMenu = new QPopupMenu();
  gridMenu->setCheckable(true);
  gridMenu->insertItem( i18n("1/1"), ID_GRID_1);
  gridMenu->insertItem( i18n("1/2"), ID_GRID_2);
  gridMenu->insertItem( i18n("1/4"), ID_GRID_4);
  gridMenu->insertItem( i18n("1/8"), ID_GRID_8);
  gridMenu->insertItem( i18n("1/16"), ID_GRID_16);
  gridMenu->insertItem( i18n("1/32"), ID_GRID_32);
  gridMenu->insertItem( i18n("1/64"), ID_GRID_64);
  gridMenu->insertItem( i18n("Off"), ID_GRID_OFF);
  gridMenu->setItemChecked(_grid_id,true);


  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu = new QPopupMenu();
  viewMenu->setCheckable(true);
  viewMenu->insertItem(i18n("&Toolbar"), ID_VIEW_TOOLBAR);
  viewMenu->insertItem(i18n("&Panel"), ID_VIEW_PANEL);
  viewMenu->insertItem(i18n("S&lider"), ID_VIEW_SLIDER);
  viewMenu->insertItem(i18n("&Statusbar"), ID_VIEW_STATUSBAR);

  viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
  viewMenu->setItemChecked(ID_VIEW_PANEL, true);
  viewMenu->setItemChecked(ID_VIEW_SLIDER, false);
  viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry globalMenu
  

  QPopupMenu * globalUserDef = new QPopupMenu();
  /* char ** alist = actionList();
     for (int i=0;i<MAXACTIONS;i++) {
     if ((alist[i]!=0) && (actionContext(i)&Addon::TREAT_GLOBAL)) globalUserDef->insertItem(alist[i],i);
     }
     connect(globalUserDef,SIGNAL(activated(int)),SLOT(slotGlobalAddon(int))); */

  char ** alist = actionList();
  char ** clist = categories();
  for (int i=0;clist[i]!=0;i++) {
    QPopupMenu * catMenu = new QPopupMenu();
    globalUserDef->insertItem(clist[i],catMenu,i);
    int * nlist = actionListByCategory(clist[i]);
    for (int k=0;nlist[k]!=-1;k++) {
      if (actionContext(nlist[k])&Addon::TREAT_GLOBAL) catMenu->insertItem(alist[nlist[k]],nlist[k]);
    }
    connect(catMenu,SIGNAL(activated(int)),SLOT(slotGlobalAddon(int)));
  }


  QPopupMenu * songUserDef = new QPopupMenu();
  /*for (int i=0;i<MAXACTIONS;i++) {
    if ((alist[i]!=0) && (actionContext(i)&Addon::TREAT_SONG)) songUserDef->insertItem(alist[i],i);
    }
    connect(songUserDef,SIGNAL(activated(int)),SLOT(slotSongAddon(int)));*/

  //char ** alist = ME->actionList();
  //char ** clist = ME->categories();
  for (int i=0;clist[i]!=0;i++) {
    QPopupMenu * catMenu = new QPopupMenu();
    songUserDef->insertItem(clist[i],catMenu,i);
    int * nlist = actionListByCategory(clist[i]);
    for (int k=0;nlist[k]!=-1;k++) {
      if (actionContext(nlist[k])&Addon::TREAT_SONG) catMenu->insertItem(alist[nlist[k]],nlist[k]);
    }
    connect(catMenu,SIGNAL(activated(int)),SLOT(slotSongAddon(int)));
  }


  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu_ = helpMenu("Brahms 0.98\n\n(c) 2000 by\nJan Wrthner & Fabian Wenzel\njan.wuerthner@gmx.de");

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // insert your popup menus with the according menubar entries in the order
  // they will appear later from left to right
  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&Edit"), editMenu);
  menuBar()->insertItem(i18n("&Grid"), gridMenu);
  menuBar()->insertItem(i18n("&View"), viewMenu);
  menuBar()->insertItem(i18n("&Song-Plugins"), songUserDef);
  menuBar()->insertItem(i18n("&Global-Plugins"), globalUserDef);

  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"), helpMenu_);

  ///////////////////////////////////////////////////////////////////
  // CONNECT THE MENU SLOTS WITH SIGNALS
  // for execution slots and statusbar messages

  connect(fileMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(fileMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(editMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(editMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(gridMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(gridMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(viewMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(viewMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));




  // Toolbar:
  // ========
  
  KIconLoader * loader = KGlobal::iconLoader();
  toolbar = new KToolBar( this );
  toolbar->insertButton( loader->loadIcon( "filenew", KIcon::Toolbar), ID_FILE_NEW, TRUE, i18n("New File"));
  toolbar->insertButton( loader->loadIcon( "fileopen", KIcon::Toolbar ), ID_FILE_OPEN, TRUE, i18n("Open a File"));
  toolbar->insertButton( loader->loadIcon( "filesave", KIcon::Toolbar ), ID_FILE_SAVE, TRUE, i18n("Save the File"));
  toolbar->insertSeparator();
  toolbar->insertSeparator();
  toolbar->insertButton( loader->loadIcon( "undo", KIcon::Toolbar), ID_EDIT_UNDO, TRUE, i18n("Undo"));
  toolbar->insertButton( loader->loadIcon( "redo", KIcon::Toolbar), ID_EDIT_REDO, TRUE, i18n("Redo"));
  toolbar->insertButton( loader->loadIcon( "editcopy", KIcon::Toolbar), ID_EDIT_COPY, TRUE, i18n("Copy"));
  toolbar->insertButton( loader->loadIcon( "editcut", KIcon::Toolbar), ID_EDIT_CUT, TRUE, i18n("Cut"));
  toolbar->insertButton( loader->loadIcon( "editpaste", KIcon::Toolbar), ID_EDIT_PASTE, TRUE, i18n("Paste"));
  toolbar->insertButton( loader->loadIcon( "addt", KIcon::Toolbar), ID_EDIT_ADD_SCORETRACK, TRUE, i18n("Add Score Track"));
  toolbar->insertSeparator();
  toolbar->insertSeparator();
  toolbar->insertButton( loader->loadIcon( "viewmag+", KIcon::Toolbar), ID_EDIT_PLUSH, TRUE, i18n("Enlarge View Horizontally"));
  toolbar->insertButton( loader->loadIcon( "viewmag-", KIcon::Toolbar), ID_EDIT_MINUSH, TRUE, i18n("Shrink View Horizontally"));
  toolbar->insertSeparator();
  toolbar->insertSeparator();

  _progressbar = new QProgressBar(100);
  // toolbar->insertWidget(137, 240, _progressbar);
  _progressbar->hide();

  toolbar->setItemEnabled( ID_EDIT_UNDO, FALSE );
  toolbar->setItemEnabled( ID_EDIT_REDO, FALSE );
  toolbar->setItemEnabled( ID_EDIT_CUT, FALSE );
  toolbar->setItemEnabled( ID_EDIT_COPY, FALSE );
  toolbar->setItemEnabled( ID_EDIT_PASTE, FALSE );
  toolbar->setItemEnabled( ID_FILE_SAVE, FALSE );

  toolbar->insertSeparator();
  toolbar->insertSeparator();
  toolbar->insertSeparator();
  toolbar->insertSeparator();
  arrTool = loader->loadIcon( "arrow", KIcon::Toolbar);
  cropTool = loader->loadIcon( "crop", KIcon::Toolbar);
  glueTool = loader->loadIcon( "wizard", KIcon::Toolbar);
  toolbar->insertButton( arrTool, ID_PANEL_ARROW, TRUE, i18n("arrow"));
  toolbar->insertButton( cropTool, ID_PANEL_CROP, TRUE, i18n("crop"));
  toolbar->insertButton( glueTool, ID_PANEL_GLUE, TRUE, i18n("glue"));
  // toolbar->insertButton( loader->loadIcon( "eraser", KIcon::Toolbar), ID_PANEL_STOP, TRUE, i18n("delete"));
  toolbar->setToggle(ID_PANEL_ARROW, TRUE);
  toolbar->setToggle(ID_PANEL_CROP, TRUE);
  toolbar->setToggle(ID_PANEL_GLUE, TRUE);
  toolbar->setButton(ID_PANEL_ARROW,TRUE);
  toolbar->setButton(ID_PANEL_CROP,FALSE);
  toolbar->setButton(ID_PANEL_GLUE,FALSE);
  
  connect(toolbar,SIGNAL(clicked(int)), SLOT(commandCallback(int)));
  connect(toolbar,SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
  

  // Panel:
  // ======

  panel = new KToolBar( this );
  panel->insertButton( loader->loadIcon( "start", KIcon::Toolbar), ID_PANEL_BACK, TRUE, i18n("Back to Start"));
  panel->insertButton( loader->loadIcon( "back", KIcon::Toolbar), ID_PANEL_REWIND, TRUE, i18n("Rewind"));
  panel->insertLined( " ", ID_PANEL_POS, SIGNAL(returnPressed()), this, SLOT(slotPosEntered()), i18n("Song Position"));
  panel->insertButton( loader->loadIcon( "forward", KIcon::Toolbar), ID_PANEL_FORWARD, TRUE, i18n("Forward"));
  panel->insertSeparator();
  panel->insertSeparator();
  // panel->insertButton( loader->loadIcon( "cycl", KIcon::Toolbar), ID_PANEL_CYCL, TRUE, i18n("cycle"));
  panel->insertButton( loader->loadIcon( "run", KIcon::Toolbar), ID_PANEL_REC, TRUE, i18n("Record"));
  panel->insertButton( loader->loadIcon( "1rightarrow", KIcon::Toolbar), ID_PANEL_PLAY, TRUE, i18n("Play"));
  panel->insertButton( loader->loadIcon( "player_pause", KIcon::Toolbar), ID_PANEL_PAUSE, TRUE, i18n("Pause"));
  panel->insertButton( loader->loadIcon( "stop", KIcon::Toolbar), ID_PANEL_STOP, TRUE, i18n("Stop"));
  panel->insertSeparator();
  panel->insertSeparator();
  panel->insertWidget(-2,38,new QLabel("range:",panel));
  _selFrame = new SelectionFrame(panel,"selframe");
  panel->insertWidget(-2,72,_selFrame);
  panel->insertSeparator();
  panel->insertSeparator();
  QLabel * tempolab  = new QLabel("tempo:",panel); panel->insertWidget(ID_EDIT_TEMPOL,48,tempolab);
  _tempobox = new QSpinBox(1,260000,1,panel); _tempobox->setPrefix(" "); panel->insertWidget(ID_EDIT_TEMPO,64,_tempobox);
  QLabel * meterlab1 = new QLabel("  meter:",panel); panel->insertWidget(ID_EDIT_ML1,48,meterlab1);
  _meterbox = new QLineEdit(panel); panel->insertWidget(ID_EDIT_M1,52,_meterbox);

  // TODO: extra panel: QSlider * posSlider = new QSlider(0,1536*8,1536,0,QSlider::Horizontal,panel); panel->insertWidget(ID_PANEL_SLIDE,24,posSlider);
  // posSlider = new KSlider(0,1536*8,1536,0,KSlider::Horizontal,panel); panel->insertWidget(ID_PANEL_SLIDE,24,posSlider);
  panel->getWidget(ID_PANEL_POS)->setFont(QFont("courier"));
  panel->getWidget(ID_PANEL_POS)->setFixedWidth(112);
  // panel->setToggle(ID_PANEL_CYCL,TRUE);
  // panel->setItemEnabled(ID_PANEL_CYCL,FALSE);
  panel->setItemAutoSized(ID_PANEL_SLIDE);
  panel->setItemEnabled(ID_PANEL_STOP,FALSE);
  panel->setItemEnabled(ID_PANEL_PAUSE,FALSE);

  connect(_tempobox,SIGNAL(valueChanged(int)),SLOT(slotTempoEntered(int)));
  connect(_meterbox,SIGNAL(returnPressed()),SLOT(slotMeterEntered()));


  connect(panel,SIGNAL(clicked(int)), SLOT(commandCallback(int)));
  connect(panel,SIGNAL(highlighted(int)), SLOT(statusCallback(int)));



  KToolBar * elements = new KToolBar( this );
  elements->insertWidget(1,10,new QLabel("",elements));
  elements->setFixedHeight(20);
  elements->show();

  _scrollbar = new QScrollBar(0,3200,1,8,0,Qt::Horizontal,elements);
  _scrollbar->setGeometry(0,2,width(),16);
  _scrollbar->show();
  connect(_scrollbar,SIGNAL(valueChanged(int)),SLOT(slotSliderChanged(int)));

  addToolBar( toolbar );
  toolbar->setBarPos(KToolBar::Top);
  toolbar->show();
  
  addToolBar( panel );
  panel->setBarPos(KToolBar::Top);
  panel->show();
  
  addToolBar( elements );
  elements->setBarPos(KToolBar::Bottom);
  enableToolBar(KToolBar::Hide,2);

  // setIndicatorWidget(new QLabel("moin",menuBar()));
  
}

void KdeMainEditor::slotSongAddon(int i) {
  if (actionList()[0] != 0) {
    performAction( i, sonG );
  }
}

void KdeMainEditor::slotGlobalAddon(int i) {
  if (actionList()[0] != 0) {
    String * dummy = new String("dummy");
    performAction( i, dummy ); // the parameter must be anything but SONG,TRACK,PART,0
    delete dummy;
  }
}

void KdeMainEditor::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n(IDS_STATUS_DEFAULT), ID_STATUS_MSG);
}


void KdeMainEditor::addRecentFile(const QString &file)
{
  if(recentFiles.find(file) == -1)
  {
    if( recentFiles.count() < 5)
    {
      recentFiles.insert(0, file);
    }
    else
    {
      recentFiles.remove(4);
      recentFiles.insert(0, file);
    }
    recentFilesMenu->clear();
    for ( int i=0 ; i < (int) recentFiles.count(); i++)
    {
      recentFilesMenu->insertItem(recentFiles.at(i));
    }
  }
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KdeMainEditor::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
  // KdeMainEditor *new_window= new KdeMainEditor();
  // new_window->show();

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));
  // if (_file_name!=0) delete _file_name;
  setFilename(strdup("untitled.bms"));
  if (sonG!=0) delete sonG;
  sonG = new Song();
  update();
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
  QString fileToOpen = KFileDialog::getOpenFileName(QDir::homeDirPath(), i18n("*.bms|Brahms"), this, i18n("Open File..."));
  if(!fileToOpen.isEmpty()) {
    setFilename( (char*) strdup(fileToOpen) );
    if (sonG!=0) delete sonG;
    sonG = Loader::load(filename(), new KdeProgress(_progressbar));
    update();
    _progressbar->reset();
  }
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileOpenRecent(int id_)
{
  slotStatusMsg(i18n("Opening file..."));
  /*	
  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    doc->openDocument(recentFiles.at(id_));
    QString caption=kapp->caption();	
    setCaption(caption+": "+doc->getTitle());
  }
  */
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
  bool saveme = true;
  if ((filename()==0) || (strcmp(filename(),"untitled.bms")==0)) {
    QString fileToSave = KFileDialog::getSaveFileName(QDir::currentDirPath(), i18n("*.bms|Brahms"), this, i18n("Save File..."));
    if(!fileToSave.isEmpty()) {
      setFilename( (char*) strdup(fileToSave) );
    } else {
      saveme = false;
    }
  }
  if (saveme) Loader::save(filename(), sonG);
  update();
  
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  QString fileToSave = KFileDialog::getSaveFileName(QDir::currentDirPath(), i18n("*.bms|Brahms"), this, i18n("Save as..."));
  if(!fileToSave.isEmpty())
  {
    setFilename( (char*) strdup(fileToSave) );
    Loader::save(filename(), sonG);
    update();
    // QFileInfo saveAsInfo(newName);
    // doc->setTitle(saveAsInfo.fileName());
    // doc->setAbsFilePath(saveAsInfo.absFilePath());
    // doc->saveDocument(newName);
    // addRecentFile(newName);

    // QString caption=kapp->caption();	
    // setCaption(caption+": "+doc->getTitle());
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


void KdeMainEditor::slotFileImportMidi() {
  slotStatusMsg(i18n("Importing Midi File..."));
  QString fileToOpen = KFileDialog::getOpenFileName(QDir::homeDirPath(), i18n("*.mid|Midi"), this, i18n("Open File..."));
  if(!fileToOpen.isEmpty()) {
    char * xy = (char*) strdup(fileToOpen);
    if (sonG!=0) delete sonG;
    sonG = Loader::loadMidi(xy, new KdeProgress(_progressbar));
    update();
    _progressbar->reset();
  }
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileExportMidi() {
  slotStatusMsg(i18n("Exporting Midi File..."));
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileAppend() {
  slotStatusMsg(i18n("Appending File..."));
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileAppendMidi() {
  slotStatusMsg(i18n("Appending Midi File..."));
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileSpecialEffects() {
  slotStatusMsg(i18n("Say hello to the authors..."));
  new KdeSpecialEffects("authors");
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    // view->print(&printer);
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  close();
  /*
    // saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KTMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
  */
}

void KdeMainEditor::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));
  if (selectArea()==TRACKS) {
    sonG->doo(new CutSelection(sonG));
  } else {
    Reference * ref = (Reference*)selectioN->first();
    if (ref!=0) {
      sonG->doo(new CutSelection(((Part*)ref->getValue())->track()));
    }
  }
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));
  sonG->doo(new CopySelection());
  
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
  selectArea(NONE);
  update();
}

void KdeMainEditor::slotEditPaste()
{
  // paste regarding the following rules:
  // in memory:     selected:     paste into:
  //   PART           PART        -> the selected PART's track
  //   PART           TRACK       -> the selected track
  //   TRACK          *           -> the song

  slotStatusMsg(i18n("Inserting clipboard contents..."));
  if (selectionMemory!=0 && selectioN!=0) {
    Reference * refMem = (Reference*)selectionMemory->first();
    Reference * refSel = (Reference*)selectioN->first();
    if (refMem!=0 && refSel!=0) {
      if        (refMem->getValue()->isTrack()) { // paste track(s) into the song: (3)
	sonG->doo(new PasteSelection(sonG, 0));
      } else if (refMem->getValue()->isA()==PART) { // check selection
	if (refSel->getValue()->isA()==PART) { // paste parts into track: (1)
	  sonG->doo(new PasteSelection(((Part*)refSel->getValue())->track(), sonG->current().ticks()));
	} else if (refSel->getValue()->isTrack()) { // paste parts into track: (2)
	  sonG->doo(new PasteSelection(((Track*)refSel->getValue()), sonG->current().ticks()));
	}
      }
    }
  }
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
  selectArea(NONE);
}

void KdeMainEditor::slotAddTrack(int i)
{
  slotStatusMsg(i18n("Adding Track..."));
  switch (i) {
  case ID_SCORE:  sonG->doo(new AddElement(new ScoreTrack(), sonG)); break;
  case ID_DRUM:   sonG->doo(new AddElement(new DrumTrack(), sonG)); break;
  case ID_MASTER: sonG->doo(new AddElement(new MasterTrack(), sonG)); break;
  case ID_AUDIO:  sonG->doo(new AddElement(new AudioTrack(), sonG)); break;
  case ID_TEXT:   sonG->doo(new AddElement(new CommentTrack(), sonG)); break;
  }
  selectArea(NONE);
  update();
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  if( viewMenu->isItemChecked(ID_VIEW_TOOLBAR))
  {
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, false);
    enableToolBar(KToolBar::Hide,0);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
    enableToolBar(KToolBar::Show,0);
  }		

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotViewPanel()
{
  slotStatusMsg(i18n("Toggle the panel..."));
  if( viewMenu->isItemChecked(ID_VIEW_PANEL))
  {
    viewMenu->setItemChecked(ID_VIEW_PANEL, false);
    enableToolBar(KToolBar::Hide,1);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_PANEL, true);
    enableToolBar(KToolBar::Show,1);
  }		

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotViewSlider()
{
  slotStatusMsg(i18n("Toggle the slider..."));
  if( viewMenu->isItemChecked(ID_VIEW_SLIDER))
  {
    viewMenu->setItemChecked(ID_VIEW_SLIDER, false);
    enableToolBar(KToolBar::Hide,2);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_SLIDER, true);
    enableToolBar(KToolBar::Show,2);
  }		

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KdeMainEditor::slotViewStatusBar() {
  slotStatusMsg(i18n("Toggle the statusbar..."));
  if( viewMenu->isItemChecked(ID_VIEW_STATUSBAR))
  {
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, false);
    enableStatusBar(KStatusBar::Hide);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);
    enableStatusBar(KStatusBar::Show);
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


void KdeMainEditor::slotGrid(int i) {
  switch (i) {
  case ID_GRID_OFF: slotStatusMsg(i18n("turn grid off")); break;
  case ID_GRID_1: slotStatusMsg(i18n("set grid to whole")); break;
  case ID_GRID_2: slotStatusMsg(i18n("set grid to half")); break;
  case ID_GRID_4: slotStatusMsg(i18n("set grid to quarter")); break;
  case ID_GRID_8: slotStatusMsg(i18n("set grid to eighths")); break;
  case ID_GRID_16: slotStatusMsg(i18n("set grid to sixteenths")); break;
  case ID_GRID_32: slotStatusMsg(i18n("set grid to 32nd")); break;
  case ID_GRID_64: slotStatusMsg(i18n("set grid to 64th")); break;
  }
  gridMenu->setItemChecked(_grid_id,false);
  _grid_id = i;
  gridMenu->setItemChecked(_grid_id,true);
  snap();
}


void KdeMainEditor::slotStatusMsg(const QString &text) {
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}


void KdeMainEditor::slotStatusHelpMsg(const QString &text) {
  statusBar()->message(text, 2000);
}


void KdeMainEditor::slotEditUndo() {
    sonG->undo();
    selectArea(NONE);
    update();
}

void KdeMainEditor::slotEditRedo() {
    sonG->redo();
    selectArea(NONE);
    update();
}

void KdeMainEditor::slotEditPlusH() {
  _pixPerTick *= 2.0;
  update();
}

void KdeMainEditor::slotEditMinusH() {
  _pixPerTick /= 2.0;
  update();
}

void KdeMainEditor::slotEditScore() {
  KdeScoreEditor * ev = new KdeScoreEditor();
  addEditor(ev);
  ev->KTMainWindow::show();
}

void KdeMainEditor::slotEditPianoRoll() {
  KdePianoRollEditor * pn = new KdePianoRollEditor();
  addEditor(pn);
  pn->KTMainWindow::show();
}

void KdeMainEditor::slotEditPreferences() {
  KdePrefs * prefs = new KdePrefs(this,"prefs");
}


void KdeMainEditor::slotSliderChanged(int i) {
  _bar_offset = i;
  update();
}

void KdeMainEditor::slotPosEntered() {
  QString qs = ((QLineEdit*) panel->getWidget(ID_PANEL_POS))->text();
  qs = qs.simplifyWhiteSpace();
  int c1 = qs.find('.');
  if (c1 != -1) {
    int c2 = qs.find('.',c1+1);
    if (c2 != -1) sonG->setCurrent(atoi(qs.left(c1)), atoi(qs.mid(c1+1,c2-c1-1)), atoi(qs.right(qs.length()-c2-1)));
  }
  panel->getWidget(ID_PANEL_POS)->clearFocus();
  update();
}

void KdeMainEditor::slotTempoEntered(int i) {
  sonG->setTempo(i);
  _tempobox->clearFocus();
  update();
}

void KdeMainEditor::slotMeterEntered() {
  QString qs = _meterbox->text();
  qs = qs.simplifyWhiteSpace();
  int c1 = qs.find('/');
  if (c1 != -1) {
    sonG->setMeter(atoi(qs.left(c1)), atoi(qs.right(qs.length()-c1-1)));
  }
  _meterbox->clearFocus();
  update();
}

void KdeMainEditor::slotTool(int id) {
  toolbar->setButton(_tool,FALSE);
  _tool = id;
  toolbar->setButton(_tool,TRUE);
  switch (id) {
  case ID_PANEL_ARROW: _right->setCursor(QCursor(arrTool,0,0)); break;
  case ID_PANEL_CROP: _right->setCursor(QCursor(cropTool,0,19)); break;
  case ID_PANEL_GLUE: _right->setCursor(QCursor(glueTool,17,6)); break;
  }
}


void KdeMainEditor::commandCallback(int id_)
{
  switch (id_) {
  case ID_FILE_NEW:
    slotFileNew();
    break;
    
  case ID_FILE_OPEN:
    slotFileOpen();
    break;
    
  case ID_FILE_SAVE:
    slotFileSave();
    break;
    
  case ID_FILE_SAVE_AS:
    slotFileSaveAs();
    break;
    
  case ID_FILE_IMPORT_MIDI:
    slotFileImportMidi();
    break;
    
  case ID_FILE_EXPORT_MIDI:
    slotFileExportMidi();
    break;
    
  case ID_FILE_APPEND:
    slotFileAppend();
    break;
    
  case ID_FILE_APPEND_MIDI:
    slotFileAppendMidi();
    break;
    
  case ID_FILE_SPECIAL_EFFECTS:
    slotFileSpecialEffects();
    break;

  case ID_FILE_CLOSE:
    slotFileClose();
    break;
	 
  case ID_FILE_QUIT:
    slotFileQuit();
    break;


  case ID_EDIT_UNDO:
    slotEditUndo();
    break;

  case ID_EDIT_UNDO_LIST:
    break;

  case ID_EDIT_REDO:
    slotEditRedo();
    break;

  case ID_EDIT_REDO_LIST:
    break;

  case ID_EDIT_CLEAR_UNDO_HISTORY:
    sonG->clearOplist();
    break;

  case ID_EDIT_CUT:
    slotEditCut();
    break;
    
  case ID_EDIT_COPY:
    slotEditCopy();
    break;
    
  case ID_EDIT_PASTE:
    slotEditPaste();
    break;
    
  case ID_EDIT_PLUSH:
    slotEditPlusH();
    break;

  case ID_EDIT_MINUSH:
    slotEditMinusH();
    break;


    case ID_EDIT_ADD_SCORETRACK:
      slotAddTrack(ID_SCORE);
      break;
      
    case ID_EDIT_ADD_DRUMTRACK:
      slotAddTrack(ID_DRUM);
      break;
 
    case ID_EDIT_ADD_MASTERTRACK:
      slotAddTrack(ID_MASTER);
      break;

    case ID_EDIT_ADD_AUDIOTRACK:
      slotAddTrack(ID_AUDIO);
      break;

    case ID_EDIT_ADD_COMMENT:
      slotAddTrack(ID_TEXT);
      break;

    case ID_EDIT_EDIT_SCORE:
      slotEditScore();
      break;

    case ID_EDIT_EDIT_PIANOROLL:
      slotEditPianoRoll();
      break;

    case ID_EDIT_MIXDOWN:
      // TODO!
      break;

    case ID_EDIT_MARK_LEFT:
      // TODO!
      break;

    case ID_EDIT_MARK_RIGHT:
      // TODO!
      break;

    case ID_EDIT_PREFERENCES:
      slotEditPreferences();
      break;


    case ID_VIEW_TOOLBAR:
         slotViewToolBar();
         break;

    case ID_VIEW_PANEL:
         slotViewPanel();
         break;

    case ID_VIEW_SLIDER:
         slotViewSlider();
         break;

    case ID_VIEW_STATUSBAR:
         slotViewStatusBar();
         break;

    case ID_GRID_1:
         slotGrid(ID_GRID_1);
         break;
    case ID_GRID_2:
         slotGrid(ID_GRID_2);
         break;
    case ID_GRID_4:
         slotGrid(ID_GRID_4);
         break;
    case ID_GRID_8:
         slotGrid(ID_GRID_8);
         break;
    case ID_GRID_16:
         slotGrid(ID_GRID_16);
         break;
    case ID_GRID_32:
         slotGrid(ID_GRID_32);
         break;
    case ID_GRID_64:
         slotGrid(ID_GRID_64);
         break;
    case ID_GRID_OFF:
         slotGrid(ID_GRID_OFF);
         break;

  case ID_PANEL_PLAY:
    // sonG->play(0,0,sonG->left().ticks(),sonG->right().ticks());
    sonG->play();
    break;
  case ID_PANEL_BACK:
    break;
  case ID_PANEL_REWIND:
    break;
  case ID_PANEL_FORWARD:
    break;
  case ID_PANEL_REC:
    break;
  case ID_PANEL_PAUSE:
    break;
  case ID_PANEL_STOP:
    break;

  case ID_PANEL_ARROW:
    slotTool(ID_PANEL_ARROW);
    break;
  case ID_PANEL_CROP:
    slotTool(ID_PANEL_CROP);
    break;
  case ID_PANEL_GLUE:
    slotTool(ID_PANEL_GLUE);
    break;

    default:
         break;
  }
  
}

void KdeMainEditor::statusCallback(int id_)
{
  // cout << "id_" << id_ << endl;
  switch (id_)
    {
    case ID_FILE_NEW:
      slotStatusHelpMsg(i18n("Creates a new document"));
      break;
      
    case ID_FILE_OPEN:
      slotStatusHelpMsg(i18n("Opens an existing document"));
      break;
      
    case ID_FILE_OPEN_RECENT:
      slotStatusHelpMsg(i18n("Opens a recently used file"));
      break;
      
    case ID_FILE_SAVE:
      slotStatusHelpMsg(i18n("Saves the actual document"));
      break;
      
    case ID_FILE_SAVE_AS:
      slotStatusHelpMsg(i18n("Saves the actual document as..."));
      break;
      
    case ID_FILE_IMPORT_MIDI:
      slotStatusHelpMsg(i18n("Imports a midi file"));
      break;
      
    case ID_FILE_EXPORT_MIDI:
      slotStatusHelpMsg(i18n("Exports a midi file"));
      break;
      
    case ID_FILE_APPEND:
      slotStatusHelpMsg(i18n("Appends a file to the actual document"));
      break;
      
    case ID_FILE_APPEND_MIDI:
      slotStatusHelpMsg(i18n("Appends a midi file to the actual document"));
      break;
      
    case ID_FILE_SPECIAL_EFFECTS:
      slotStatusHelpMsg("Hans hrt hinterm Haus hundert Hasen husten");
      break;
      
    case ID_FILE_CLOSE:
         slotStatusHelpMsg(i18n("Closes the actual document"));
         break;

    case ID_FILE_QUIT:
         slotStatusHelpMsg(i18n("Quits the application"));
         break;

    case ID_EDIT_UNDO:
         slotStatusHelpMsg(i18n(sonG->currentUndoText()));
         break;

    case ID_EDIT_REDO:
         slotStatusHelpMsg(i18n(sonG->currentRedoText()));
         break;

    case ID_EDIT_CUT:
         slotStatusHelpMsg(i18n("Cuts the selected section and puts it to the clipboard"));
         break;

    case ID_EDIT_COPY:
         slotStatusHelpMsg(i18n("Copies the selected section to the clipboard"));
         break;

    case ID_EDIT_PASTE:
         slotStatusHelpMsg(i18n("Pastes the clipboard contents to actual position"));
         break;

    case ID_EDIT_ADD_SCORETRACK:
      slotStatusHelpMsg(i18n("Adds a scoretrack to the document"));
      break;
      
    case ID_EDIT_ADD_DRUMTRACK:
      slotStatusHelpMsg(i18n("Adds a drumtrack to the document"));
      break;
 
    case ID_EDIT_ADD_MASTERTRACK:
      slotStatusHelpMsg(i18n("Adds a mastertrack to the document"));
      break;

    case ID_EDIT_ADD_AUDIOTRACK:
      slotStatusHelpMsg(i18n("Adds an audiotrack to the document"));
      break;

    case ID_EDIT_ADD_COMMENT:
      slotStatusHelpMsg(i18n("Adds a comment to the document"));
      break;

    case ID_EDIT_EDIT_SCORE:
      slotStatusHelpMsg(i18n("Opens the score editor"));
      break;

    case ID_EDIT_MIXDOWN:
      slotStatusHelpMsg(i18n("Mixes down tracks to a new track"));
      break;

    case ID_EDIT_MARK_LEFT:
      slotStatusHelpMsg(i18n("Sets the left marker"));
      break;

    case ID_EDIT_MARK_RIGHT:
      slotStatusHelpMsg(i18n("Sets the right marker"));
      break;

    case ID_EDIT_PREFERENCES:
      slotStatusHelpMsg(i18n("Opens the preferences"));
      break;

    case ID_VIEW_TOOLBAR:
         slotStatusHelpMsg(i18n("Enables/disables the toolbar"));
         break;

    case ID_VIEW_STATUSBAR:
         slotStatusHelpMsg(i18n("Enables/disables the statusbar"));
         break;

    default:
         break;
  }
}

void KdeMainEditor::initSong() {

  ScoreTrack * tr = new ScoreTrack();
  sonG->doo(new AddElement(tr,sonG));
  sonG->doo(new AddElement(new ScoreTrack(),sonG));
  sonG->doo(new AddElement(new ScoreTrack(),sonG));
  sonG->doo(new AddElement(new ScoreTrack(),sonG));


  Part * pt = new Part(tr);
  sonG->doo(new AddElement(pt,tr));

  sonG->doo(new AddElement(new Note(68,30,192,0,0),pt));
  sonG->doo(new AddElement(new Note(70,100,192,192,0),pt));
  sonG->doo(new AddElement(new Note(72,120,1536,384,0),pt));
  
}

QWidget * KdeMainEditor::left() { return _left; }

QWidget * KdeMainEditor::right() { return _right; }

int KdeMainEditor::trackHeight() { return _track_height; }

double KdeMainEditor::pixPerTick() { return _pixPerTick; }

int KdeMainEditor::barOffset() { return _bar_offset; }


#endif
