#ifndef _KDEMAINEDITOR_H_
#define _KDEMAINEDITOR_H_

#include <ktmainwindow.h>
#include <kapp.h>
#include <qstrlist.h>
#include <qscrollview.h>
#include <qsplitter.h>
#include <qpixmap.h>

#include "../../prMainEditor.h"

class QPopupMenu;
class QScrollBar;
class QProgressBar;
class QSpinBox;
class KAccel;
class KToolBar;
class HeadLine;
class SelectionFrame;
class TimeBar;

enum SelectArea { NONE, TRACKS, PARTS };

class KdeMainEditor : public KTMainWindow, public PrMainEditor
{
  Q_OBJECT

 private:
  KApplication * _app;
  int            _argc;
  char **        _argv;
  SelectArea     _select_area;
  KToolBar *     panel;
  QScrollView *  _scroller;
  QSplitter *    _splitter;
  QWidget *      _left; 
  QWidget *      _child;
  QWidget *      _right;
  QScrollBar *   _scrollbar;
  QProgressBar * _progressbar;
  HeadLine *     _headline;
  TimeBar *      _timebar;
  SelectionFrame * _selFrame;
  QSpinBox *     _tempobox;
  QLineEdit *    _meterbox;
  int            _tool;
  QPixmap        cropTool;
  QPixmap        glueTool;
  QPixmap        arrTool;
  QCursor    *   arrcursor;

  

  char * _meter_text;
  char * _caption_text;
  char * _position_text;
  int _grid_id;
  int _track_width;
  int _track_height;
  double _pixPerTick;
  int _bar_offset;

  void initKeyAccel();
  void initMenuBar();
  void initStatusBar();
  void initView();
  void initSong();

  /** contains the recently used filenames */
  QStrList recentFiles;
  
  /** the configuration object of the application */
  // KConfig *config;
  /** the key accelerator container */
  KAccel *keyAccel;
  /** file_menu contains all items of the menubar entry "File" */
  QPopupMenu *fileMenu;
  /** the recent file menu containing the last five opened files */
  QPopupMenu *recentFilesMenu;
  /** edit_menu contains all items of the menubar entry "Edit" */
  QPopupMenu *editMenu;
  /** grid_menu contains all items of the menubar entry "Grid" */
  QPopupMenu *gridMenu;
  /** view_menu contains all items of the menubar entry "View" */
  QPopupMenu *viewMenu;
  /** help_menu contains all items of the menubar entry "Help" */
  QPopupMenu *helpMenu_;

  KToolBar * toolbar;

 public:
  KdeMainEditor(char * name, KApplication * app, int argc, char ** argv);
  virtual void run();
  virtual void update();
  virtual void gui(GuiEvents,bool);

  int trackHeight();
  double pixPerTick();
  int barOffset();
  SelectArea selectArea();
  void selectArea(SelectArea);

  QWidget * left();
  QWidget * right();
  void displaySongProperties();
  int snap();

  void addRecentFile(const QString &file);
  int tool();

  public slots:
    void commandCallback(int id_);
    void statusCallback(int id_);
    void slotFileNewWindow();
    void slotFileNew();
    void slotFileOpen();
    void slotFileOpenRecent(int id_);
    void slotFileSave();
    void slotFileSaveAs();
    void slotFileImportMidi();
    void slotFileExportMidi();
    void slotFileAppend();
    void slotFileAppendMidi();
    void slotFileSpecialEffects();
    void slotFileClose();
    void slotFilePrint();
    void slotFileQuit();
    void slotEditUndo();
    void slotEditRedo();
    void slotEditPlusH();
    void slotEditMinusH();
    void slotEditCut();
    void slotEditCopy();
    void slotEditPaste();
    void slotEditScore();
    void slotEditPianoRoll();
    void slotEditPreferences();
    void slotAddTrack(int);
    
    void slotViewToolBar();
    void slotViewPanel();
    void slotViewSlider();
    void slotViewStatusBar();

    void slotSliderChanged(int);
    void slotPosEntered();
    void slotTempoEntered(int);
    void slotMeterEntered();

    void slotSongAddon(int);
    void slotGlobalAddon(int);
    void slotGrid(int);
    void slotStatusMsg(const QString &text);
    void slotStatusHelpMsg(const QString &text);
    
    void slotTool(int);

    void resizeEvent( QResizeEvent * );
};

#endif
