#ifndef _QTMAINEDITOR_CPP_
#define _QTMAINEDITOR_CPP_

#include <stdlib.h>
#include <iostream.h>
#include <math.h>

#include "../../song.h"
#include "../../scoreTrack.h"
#include "../../drumTrack.h"
#include "../../masterTrack.h"
#include "../../audioTrack.h"
#include "../../textTrack.h"
#include "../../part.h"
#include "../../addElement.h"
#include "../../vector.h"
#include "../../str.h"
#include "../../note.h"
#include "../../loader.h"

#include "qtMainEditor.h"
#include "qtFactory.h"
#include "qtPartEditor.h"

extern Song * sonG;

QtMainEditor::QtMainEditor(int argc, char ** argv) : MainEditor(argc,argv) {
  if (argc < 3) {
    sonG = new Song();
  } else {
    char * nam = argv[2];
    sonG = Loader::load(nam);
  }

}

void QtMainEditor::run() {
  char line[400];
  char * cmd;
  bool exit = false;
  bool debug = false;

  update();
  while (!exit) {
    cout << "[ " << sonG->total() << " ] $ ";
    cin.getline(line,400,'\n');
    Vector vec = Vector(line);
    cmd = ((String*) (vec)[0])->getValue();
    // if (vec[0]=="hallo") cout << "ok" << endl;
    if (debug) cout << vec << endl;
    if (strcmp(cmd,"")==0) { update(); }
    if (strcmp(cmd,"help")==0) {
      cout << "help            - shows this list" << endl;
      cout << "copy            - copies the song (test)" << endl;
      cout << "list            - lists the song" << endl;
      cout << "debug           - switches debug mode on and off" << endl;
      cout << "add scoretrack  - adds a track to the song" << endl;
      cout << "add drumtrack   - adds a track to the song" << endl;
      cout << "add mastertrack - adds a track to the song" << endl;
      cout << "add audiotrack  - adds a track to the song" << endl;
      cout << "add texttrack   - adds a track to the song" << endl;
      cout << "add part n      - adds a part to track n" << endl;
      cout << "add note n m    - adds a note to part n, track m" << endl;
      cout << "   [pos pitch len vel]" << endl;
      cout << "oplist          - lists the operations in the stack of the song" << endl;
      cout << "undo            - undoes one operation" << endl;
      cout << "redo            - redoes one operation" << endl;
      cout << "dump            - dumps all elements" << endl;
      cout << "clear           - delete song" << endl;
      cout << "load <fname>    - load a song" << endl;
      cout << "save <fname>    - save a song" << endl;
      cout << "im   <fname>    - load a midi song" << endl;
      cout << "play            - play song" << endl;
      cout << "edit n          - edit track n" << endl;
      cout << "tempo n         - sets the tempo" << endl;
    }
    if (strcmp(cmd,"exit")==0) exit = true;
    if (strcmp(cmd,"copy")==0) {
      // Track * tr = (Track*) sonG->get(0)->copy();
      Song * s1 = (Song*) sonG->copy();
    }
    if (strcmp(cmd,"clear")==0) { delete sonG; sonG = 0; }
    if (strcmp(cmd,"list")==0) { cout << *sonG << endl; }
    if (strcmp(cmd,"debug")==0) { if (debug) { debug = false; } else { cout << vec << endl; debug = true; } }
    if (strcmp(cmd,"undo")==0) { sonG->undo(); }
    if (strcmp(cmd,"play")==0) { sonG->play(); }
    if (strcmp(cmd,"edit")==0) {
      int par = atoi(((String*) (vec)[1])->getValue()) - 1; // track
      Track * tr = (Track*)sonG->get(par);
      if (tr) {
	QtPartEditor * tpe = new QtPartEditor(tr);
	delete tpe;
	update();
      }
    }
    if (strcmp(cmd,"tempo")==0) {
      int par = atoi(((String*) (vec)[1])->getValue());
      if (par > 20 && par < 1000) sonG->setTempo(par);
    }
    if (strcmp(cmd,"load")==0) {
      char * par1 = ((String*) (vec)[1])->getValue();
      if (par1!=0) {
	if (sonG!=0) delete sonG;
	sonG = Loader::load(par1);
      }
    }
    if (strcmp(cmd,"save")==0) {
      char * par1 = ((String*) (vec)[1])->getValue();
      if (par1!=0) {
	Loader::save(par1,sonG);
      }
    }
    if (strcmp(cmd,"oplist")==0) { sonG->oplist(); }
    if (strcmp(cmd,"redo")==0) { sonG->redo(); }
    if (strcmp(cmd,"dump")==0) { sonG->dump(); }
    if (strcmp(cmd,"add")==0) {
      char * par1 = ((String*) (vec)[1])->getValue();
      if (strcmp(par1,"st")==0) {
	sonG->doo(new AddElement(new ScoreTrack(),sonG));
	cout << "added scoretrack to song" << endl;
      } else if (strcmp(par1,"drumtrack")==0) {
	sonG->doo(new AddElement(new DrumTrack(),sonG));
	cout << "added drumtrack to song" << endl;
      } else if (strcmp(par1,"mastertrack")==0) {
	sonG->doo(new AddElement(new MasterTrack(),sonG));
	cout << "added mastertrack to song" << endl;
      } else if (strcmp(par1,"audiotrack")==0) {
	sonG->doo(new AddElement(new AudioTrack(),sonG));
	cout << "added audiotrack to song" << endl;
      } else if (strcmp(par1,"texttrack")==0) {
	sonG->doo(new AddElement(new TextTrack(),sonG));
	cout << "added texttrack to song" << endl;
      } else if (strcmp(par1,"part")==0) {
	int par2 = atoi(((String*) (vec)[2])->getValue()) - 1;
	if (par2<0) par2 = 0;
	Track * tr = (Track*)sonG->get(par2);
	if (tr) {
	  Part * pt = new Part();
	  pt->setMeter(sonG->meter0(),sonG->meter1());
	  sonG->doo(new AddElement(pt,tr));
	  cout << "added part to track " << par2+1 << endl;
	} else cout << "*** no tracks" << endl;
      } else if (strcmp(par1,"note")==0) {
	int par2 = atoi(((String*) (vec)[2])->getValue()) - 1; // part
	int par3 = atoi(((String*) (vec)[3])->getValue()) - 1; // track
	// pitch, vel, len, pos, enh
	int pitch = 76;
	int vel = 86;
	int len = 192;
	int pos = 0;
	int enh = 0;
	int num = vec.size();
	cout << "num: " << num << endl;
	if (num>4) { pos = atoi(((String*) (vec)[4])->getValue()); }
	if (num>5) { pitch = atoi(((String*) (vec)[5])->getValue()); }
	if (num>6) { len = atoi(((String*) (vec)[6])->getValue()); }
	if (num>7) { vel = atoi(((String*) (vec)[7])->getValue()); }
	if (par2<0) par2 = 0;
	if (par3<0) par3 = 0;
	Track * tr = (Track*)sonG->get(par3);
	if (tr) {
	  Part * pt = (Part*)tr->get(par2);
	  if (pt) {
	    // Note * n = new Note(pitch, vel, len, Position(pos), enh);
	    Note * n = new Note();
	    sonG->doo(new AddElement(n,pt));
	    cout << "added note to part " << par2+1 << " of track " << par3+1 << endl;
	  } else cout << "*** no parts" << endl;
	} else cout << "*** no tracks" << endl;
      }
    }
  }
  
}

void QtMainEditor::update() {
  cout << "\ec" << endl;
  cout << "Welcome to the Brahms qt console\n" << endl;
  if (sonG) {
    // cout << sonG << endl;
    cout << "Title: " << sonG->title() << " (" << sonG->author() << ", " << sonG->comment() << ")" << endl;
    cout << "(v" << sonG->version() << ")" << endl;
    cout << "Tempo: " << sonG->tempo() << "  Meter: " << sonG->meter0() << "/" << sonG->meter1() << endl;
    cout << endl;
    int imax = sonG->size();
    Track * tr = 0;
    Part * pt = 0;
    for (int i=0; i<imax; i++) {
      tr = (Track*) sonG->get(i);
      cout << "Track " << i+1 << " : ";
      int kmax = tr->size();
      for (int k=0; k<kmax; k++) {
	// for (Part * pt = (Part*) tr->first(); pt != 0; pt = (Part*) Element::next(pt)) {
	pt = (Part*) tr->get(k);
	int len = pt->size();
	cout << "[Part " << k+1 << ", " << len << " events] ";
      }
      cout << endl;
    }
  } else {
    cout << "no song in memory" << endl;
  }
  cout << endl;
}



#endif
