#ifndef _SONG_CPP_
#define _SONG_CPP_

#include <iostream.h>
#include <string.h>
#include <fstream.h>
#include <math.h>

#include "song.h"
#include "part.h"
#include "operation.h"
#include "vector"
#include "table.h"
#include "str.h"
#include "track.h"
#include "loader.h"
#include "player.h"
#include "prProgress.h"
#include "prMainEditor.h"

extern char * getcmd(char*);
extern PrMainEditor * mainEditor;
extern Table * selectioN;
extern Table * selectionMemory;
extern Song * sonG;

Song::Song()
  : _title("untitled"), _author("-"), _comment("-"), _tempo(120), _meter0(4), _meter1(4), _left((long)0), _right((long)12288), _current(0), _master(0), _version(0),
    _part_appearance(TRACKNAME)
{
  _type = SONG;
  _player = new Player(this);
  sonG = this;
}

Song::Song(double v)
  : _title("untitled"), _author("-"), _comment("-"), _tempo(120), _meter0(4), _meter1(4), _left((long)0), _right((long)12288), _current(0), _master(0), _version(v),
    _part_appearance(TRACKNAME)
{
  _type = SONG;
  _player = new Player(this);
  sonG = this;
}

Song::Song(Song * e)
  : Compound(e,SONG), _title("untitled"), _author("-"), _comment("-"), _tempo(120), _meter0(4), _meter1(4), _left((long)0), _right((long)12288), _current(0), _master(0),
    _part_appearance(TRACKNAME)
{
  _title = strdup(e->title());
  _author = strdup(e->author());
  _comment = strdup(e->comment());
  _tempo = e->tempo();
  _meter0 = e->meter0();
  _meter1 = e->meter1();
  _left = e->left();
  _right = e->right();
  _current = e->current();
  _version = e->version();
  if (e->master()!=0) _master = (Part*) e->master()->copy();
  _player = new Player(this);
  sonG = this;
}

Song::~Song() { delete _player; }

void Song::updateGui(int changes) {
  if (changes!=-1) mainEditor->gui(CHANGES,changes);
  if (opHistory.empty()) mainEditor->ui(UNDO,false);
  else                   mainEditor->ui(UNDO,true);
  if (opFuture.empty()) mainEditor->ui(REDO,false);
  else                  mainEditor->ui(REDO,true);
  if (selectioN->empty()) mainEditor->ui(SELECTIONS,false);
  else                    mainEditor->ui(SELECTIONS,true);
  if (selectionMemory->empty()) mainEditor->ui(MEMORY,false);
  else                          mainEditor->ui(MEMORY,true);
}

void Song::doo(Operation * op) {
  if (op->isToBeUndone()) {
    
    opHistory.add(op);
    if (opFuture.size()>0) {
      opFuture.scratch();
    }
    if (opHistory.size() > mainEditor->undoMemory()) {
      Operation * del = (Operation*) opHistory.cutFirst();
      delete del;
    }
    updateGui(true);
  } else {
    updateGui();
    delete op;
  }
}

void Song::undo() {
  if (!opHistory.empty()) {
    Operation * op = (Operation*) opHistory.cutLast();
    op->undo();
    opFuture.push(op);
    updateGui(true);
  } 
}

void Song::redo() {
  if (!opFuture.empty()) {
    Operation * op = (Operation*) opFuture.cutFirst();
    op->redo();
    opHistory.add(op);
    updateGui(true);
  }
  if (opFuture.empty()) mainEditor->gui(REDO,false);
}

void Song::oplist() {
  cout << "History:\n" << opHistory << endl;
  cout << "Future:\n" << opFuture << endl;
}

void Song::clearOplist() {
  opHistory.scratch();
  opFuture.scratch();
  updateGui();
}

Operation * Song::currentOp() {
  return (Operation*) opHistory.last();
}

const char * Song::currentUndoText() {
  Operation * op = (Operation*) opHistory.last();
  if (op!=0) return op->description();
  else return " ";
}

const char * Song::currentRedoText() {
  Operation * op = (Operation*) opFuture.first();
  if (op!=0) return op->description();
  else return "-";

}

char * Song::title() { return _title; }

char * Song::author() { return _author; }

char * Song::comment() { return _comment; }

PartApp Song::partAppearance() { return _part_appearance; }

int Song::tempo() { return _tempo; }

int Song::meter0() { return _meter0; }

int Song::meter1() { return _meter1; }

Position Song::left() { return _left; }

Position Song::right() { return _right; }

Position Song::current() { return _current; }

Part * Song::master() { return _master; }



void Song::setTitle(char * t) { _title = t; }

void Song::setAuthor(char * a) { _author = a; }

void Song::setComment(char * c) { _comment = c; }

void Song::setPartAppearance(PartApp i) { _part_appearance = i; }

void Song::setTempo(int t) { _tempo = t; }

void Song::setMeter(int m0, int m1) { setMeter0(m0); setMeter1(m1); }

void Song::setMeter0(int m0) { _meter0 = m0; }

void Song::setMeter1(int m1) { _meter1 = m1; }

void Song::setLeft(Position p) { _left = p; }

void Song::setRight(Position p) { _right = p; }

void Song::setCurrent(Position p) { _current = p; }

void Song::setCurrent(int a, int b, int c) {
  _current.set(a,b,c,_master,_meter0,_meter1);
}

void Song::setMaster(Part * pt) { _master = pt; }

Player * Song::player() { return _player; }


void Song::bbt(int & bar, int & beat, int & tick, Position pos) {
  // int i = int(pow(2,_meter1));
  pos.gBBT(bar,beat,tick,_master,_meter0,_meter1);
}

void Song::play(PrProgress * progress, Part * part, long left, long right) {
  _player->play(progress,part,left,right);
}

double Song::version() { return _version; }


ostream & Song::print(int dep, ostream & s) {
  s << spc(dep) << "<SONG>" << endl;
  s << spc(dep) << "<HEAD title=\"" << _title << "\" author=\"" << _author << "\" comment=\"" << _comment << "\" ";
  s << "tempo=\"" << _tempo << "\" meter0=\"" << _meter0 << "\" meter1=\"" << _meter1 << "\" ";
  s << "left=\"" << _left << "\" right=\"" << _right << "\" tracks=\"" << size() << "\" />" << endl;
  printContent(dep,s);
  s << spc(dep) << "</SONG>" << endl;
  return s;
}

void Song::flush(char* c) {
  cout << c << "SONG" << endl;
}


Element * Song::copy() {
  Song * sng = new Song(this);
  return sng;
}

/** head:
    int tempo,
    char* title,
    char* comment,
    char* author,
    Position left,
    Position right,
    int meter0,
    int meter1,
    int tracks="4" (obsolete)
*/

Element * Song::load(ifstream * inPtr, double version, PrProgress * progress) {
  Song * song = new Song(version);
  if (version>=98.0) {
    Table * attr = Loader::getAttributes("SONG",inPtr);
    attr->scratch();
    delete attr;
  }
  Table * head = Loader::getAttributes("HEAD",inPtr);
  int num = 0;
  if (head) {
    String * val = 0;
    val = (String*) head->getEntry("tempo"); if (val) song->setTempo( atoi(val->getValue()) );
    val = (String*) head->getEntry("title"); if (val) song->setTitle( strdup(val->getValue()) );
    val = (String*) head->getEntry("comment"); if (val) song->setComment( strdup(val->getValue()) );
    val = (String*) head->getEntry("author"); if (val) song->setAuthor( strdup(val->getValue()) );
    val = (String*) head->getEntry("left"); if (val) song->setLeft( Position(atoi(val->getValue())) );
    val = (String*) head->getEntry("right"); if (val) song->setRight( Position(atoi(val->getValue())) );
    val = (String*) head->getEntry("meter0"); if (val) song->setMeter0( atoi(val->getValue()) );
    val = (String*) head->getEntry("meter1"); if (val) song->setMeter1( atoi(val->getValue()) );
    val = (String*) head->getEntry("tracks"); if (val) num = atoi(val->getValue());

    /* wech!
    song->setTempo( atoi(((String*) head->getEntry("tempo"))->getValue()) );
    song->setTitle( ((String*) head->getEntry("title"))->getValue() );
    song->setComment( ((String*) head->getEntry("comment"))->getValue() );
    song->setAuthor( ((String*) head->getEntry("author"))->getValue() );
    song->setLeft( Position(atoi(((String*) head->getEntry("left"))->getValue())) );
    song->setRight( Position(atoi(((String*) head->getEntry("right"))->getValue())) );
    song->setMeter0( atoi(((String*) head->getEntry("meter0"))->getValue()) );
    song->setMeter1( atoi(((String*) head->getEntry("meter1"))->getValue()) );
    int num = atoi(((String*) head->getEntry("tracks"))->getValue()); */

    head->scratch();
    delete head;
  }
  if (version>=98.0) {
    Element::loadContent(song,"/SONG","TRACK",&Track::load,inPtr,progress,num);
  } else {
    song->setMeter1(pow(2,song->meter1()));
    Element::loadContent(song,"/DOCUMENT","TRACK",&Track::load,inPtr,progress,num);
  }
  return song;
}


#endif
