#ifndef _SONG_H_
#define _SONG_H_

#include "compound.h"
#include "position.h"
#include "operation.h"
#include "vector.h"

class PrProgress;
class Player;

enum PartApp { CLEAN, TRACKNAME, INSTRUMENT, EVENTS };

class Song : public Compound
{
 private:

  char * _title;
  char * _author;
  char * _comment;
  PartApp _part_appearance;
  int _tempo;
  int _meter0;
  int _meter1;
  Position _left; // left locator
  Position _right; // right locator
  Position _current; // current position pointer
  Part * _master;
  Vector opHistory;
  Vector opFuture;
  double _version;
  Player * _player;

 public:

  Song();
  Song(double);
  Song(Song*);
  ~Song();

  void doo(Operation*);
  void undo();
  void redo();
  void oplist();
  void clearOplist();
  Operation * currentOp();
  const char * currentUndoText();
  const char * currentRedoText();
  void updateGui(int changes=-1);

  char * title();
  char * author();
  char * comment();
  PartApp partAppearance();
  int tempo();
  int meter0();
  int meter1();
  Position left();
  Position right();
  Position current();
  Part * master();

  void setTitle(char*);
  void setAuthor(char*);
  void setComment(char*);
  void setPartAppearance(PartApp);
  void setTempo(int);
  void setMeter(int,int);
  void setMeter0(int);
  void setMeter1(int);
  void setLeft(Position);
  void setRight(Position);
  void setCurrent(Position);
  void setCurrent(int,int,int);
  void setMaster(Part*);
  Player * player();

  void bbt(int & bar, int & beat, int & tick, Position pos);
  void play(PrProgress * progress = 0, Part * part = 0, long left = 0, long right = 0);
  double version();
  
  virtual ostream & print(int,ostream&);
  virtual void flush(char*);
  virtual Element * copy();
  
  static Element * load(ifstream*,double,PrProgress*);
};

#endif
