#ifndef _TRACK_CPP_
#define _TRACK_CPP_

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string.h>
#include "track.h"
#include "str.h"
#include "table.h"
#include "element.h"
#include "loader.h"
#include "scoreTrack.h"
#include "drumTrack.h"
#include "masterTrack.h"
#include "audioTrack.h"
#include "commentTrack.h"
#include "part.h"
#include "song.h"
#include "convertTrack.h"
#include "prTrack.h"

extern Song * sonG;
extern char * getcmd(char*);

Track::Track() : _name(new String("new track")), _mute(false), _lock(false) {
  _type = TRACK;
}

Track::Track(Track * e, Type t) : Compound(e,t) {
  if (e) {
    setName((String*)e->name()->copy());
    _mute = e->mute();
    _lock = e->lock();
  }
}

Track::~Track() {
  if (_name!=0) delete _name;
  if (_pr!=0) _pr->erase();
  cout << "scratching track" << endl;
  scratch();
  cout << "ok" << endl;
}

String * Track::name() { return _name; }

bool Track::mute() { return _mute; }

bool Track::lock() { return _lock; }

void Track::setName(String * str) {
  if (_name!=0) { delete _name; }
  _name = str;
}

void Track::setName(char * c) {
  if (_name!=0) { delete _name; }
  _name = new String(c);
}

void Track::setMute(bool m) { _mute = m; }

void Track::setLock(bool l) { _lock = l; }

PrTrack * Track::presentation() { return _pr; }


void Track::hide() {
  for (Part * pt = (Part*) this->first(); pt != 0; pt = (Part*) this->next(pt)) pt->hide();
  if (_pr!=0) _pr->hide();
}

void Track::show() {
  for (Part * pt = (Part*) this->first(); pt != 0; pt = (Part*) this->next(pt)) pt->show();
  if (_pr!=0) _pr->show();
}

ostream & Track::print(int dep, ostream & s) {
  s << spc(dep) << "<TRACK type=\"" << isA() << "\" name=\"" << _name->getValue() << "\" mute=\"" << _mute << "\" lock=\"" << _lock << "\" >" << endl;
  printOptions(dep,s);
  printContent(dep,s);
  s << spc(dep) << "</TRACK>" << endl;
  return s;
}

void Track::flush(char* c) {
  cout << c << "TRACK" << endl;
}

/*Element * Track::copy() {
  Track * sng = new Track(this);
  return sng;
  }*/

/* <TRACK type="0" name="Sopran" mute="0" lock="0" >
<OPTIONS program="0" channel="0" output="2" volume="0" transpose="0" delay="0" />
*/

Element * Track::load(char * line, ifstream *& inPtr, Element * parent) {
  Table * attr = Loader::getAttributes("TRACK",line);
  //cout << *attr << endl;

  Track * tr = 0;
    if (attr->getEntry("type") && attr->getEntry("name") && attr->getEntry("mute") && attr->getEntry("lock")) {
    int type = atoi(((String*) attr->getEntry("type"))->getValue());
    char * name = ((String*) attr->getEntry("name"))->getValue();
    int mute = atoi(((String*) attr->getEntry("mute"))->getValue());
    int lock = atoi(((String*) attr->getEntry("lock"))->getValue());
    
    switch (type) {
    case 0: tr = new ScoreTrack(); ((ScoreTrack*)tr)->loadOpt(inPtr); break;
    case 1: tr = new DrumTrack(); ((DrumTrack*)tr)->loadOpt(inPtr); break;
    case 2: tr = new MasterTrack(); ((MasterTrack*)tr)->loadOpt(inPtr); break;
    case 3: tr = new AudioTrack(); ((AudioTrack*)tr)->loadOpt(inPtr); break;
    case 4: tr = new CommentTrack(); ((CommentTrack*)tr)->loadOpt(inPtr); break;
    }
    tr->setName(name);
    tr->setMute(mute);
    tr->setLock(lock);
    attr->scratch();
    delete attr;
    
    Element::loadContent(tr,"/TRACK","PART",&Part::load,inPtr);
    // problem: parts have to know about the tracks, they belong to (this does not fit into the general algorithm)
    // thus: OBSOLETE! 081000
    // for (Part * pt = (Part*) tr->first(); pt != 0; pt = (Part*) tr->next(pt)) pt->setTrack(tr);
  }
  return tr;
}

#endif
